# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

# pretrained infos

pretrained_xlm_roberta_infos = {
    "default-xlm-roberta": {
        "config": "https://huggingface.co/xlm-roberta-base/resolve/main/config.json",
        "vocab": "https://huggingface.co/xlm-roberta-base/resolve/main/sentencepiece.bpe.model",
    },
    "xlm-roberta-base": {
        "config": "https://huggingface.co/xlm-roberta-base/resolve/main/config.json",
        "vocab": "https://huggingface.co/xlm-roberta-base/resolve/main/sentencepiece.bpe.model",
        "weight": "https://huggingface.co/xlm-roberta-base/resolve/main/pytorch_model.bin",
    },
    "xlm-roberta-large": {
        "config": "https://huggingface.co/xlm-roberta-large/resolve/main/config.json",
        "vocab": "https://huggingface.co/xlm-roberta-large/resolve/main/sentencepiece.bpe.model",
        "weight": "https://huggingface.co/xlm-roberta-large/resolve/main/pytorch_model.bin",
    },
    "xlm-roberta-large": {
        "config": "https://huggingface.co/xlm-roberta-large/resolve/main/config.json",
        "vocab": "https://huggingface.co/xlm-roberta-large/resolve/main/sentencepiece.bpe.model",
        "weight": "https://huggingface.co/xlm-roberta-large/resolve/main/pytorch_model.bin",
    },
    "xlm-roberta-large-finetuned-conll03-english": {
        "config": "https://huggingface.co/xlm-roberta-large-finetuned-conll03-english/resolve/main/config.json",
        "vocab": "https://huggingface.co/xlm-roberta-large-finetuned-conll03-english/resolve/main/sentencepiece.bpe.model",
        "weight": "https://huggingface.co/xlm-roberta-large-finetuned-conll03-english/resolve/main/pytorch_model.bin",
    },
    "xlm-roberta-xl": {
        "config": "https://huggingface.co/facebook/xlm-roberta-xl/resolve/main/config.json",
        "vocab": "https://huggingface.co/facebook/xlm-roberta-xl/resolve/main/sentencepiece.bpe.model",
        "weight": "https://huggingface.co/facebook/xlm-roberta-xl/resolve/main/pytorch_model.bin",
    },
}

import unitorch.cli.models.xlm_roberta.modeling
import unitorch.cli.models.xlm_roberta.processing
from unitorch.cli.models.xlm_roberta.modeling import (
    XLMRobertaForClassification,
    XLMRobertaXLForClassification,
)
from unitorch.cli.models.xlm_roberta.processing import XLMRobertaProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

# pretrained infos
pretrained_t5_infos = {
    "default-t5": {
        "config": "https://huggingface.co/t5-base/resolve/main/config.json",
        "vocab": "https://huggingface.co/t5-base/resolve/main/spiece.model",
    },
    "t5-base": {
        "config": "https://huggingface.co/t5-base/resolve/main/config.json",
        "vocab": "https://huggingface.co/t5-base/resolve/main/spiece.model",
        "weight": "https://huggingface.co/t5-base/resolve/main/pytorch_model.bin",
    },
    "t5-small": {
        "config": "https://huggingface.co/t5-small/resolve/main/config.json",
        "vocab": "https://huggingface.co/t5-small/resolve/main/spiece.model",
        "weight": "https://huggingface.co/t5-small/resolve/main/pytorch_model.bin",
    },
    "t5-large": {
        "config": "https://huggingface.co/t5-large/resolve/main/config.json",
        "vocab": "https://huggingface.co/t5-large/resolve/main/spiece.model",
        "weight": "https://huggingface.co/t5-large/resolve/main/pytorch_model.bin",
    },
}

import unitorch.cli.models.t5.modeling
import unitorch.cli.models.t5.processing
from unitorch.cli.models.t5.modeling import T5ForGeneration
from unitorch.cli.models.t5.processing import T5Processor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from diffusers.schedulers import (
    DPMSolverSDEScheduler,
    UniPCMultistepScheduler,
    EulerAncestralDiscreteScheduler,
    EulerDiscreteScheduler,
    FlowMatchEulerDiscreteScheduler,
)

Schedulers = {
    "DPM++SDE": DPMSolverSDEScheduler,
    "UniPC++": UniPCMultistepScheduler,
    "FlowMatchEuler": FlowMatchEulerDiscreteScheduler,
    "EulerAncestral": EulerAncestralDiscreteScheduler,
    "Euler": EulerDiscreteScheduler,
}

import unitorch.cli.pipelines.blip
import unitorch.cli.pipelines.bloom
import unitorch.cli.pipelines.bria
import unitorch.cli.pipelines.detr
import unitorch.cli.pipelines.dpt
import unitorch.cli.pipelines.stable
import unitorch.cli.pipelines.stable_xl
import unitorch.cli.pipelines.stable_3
import unitorch.cli.pipelines.llama
import unitorch.cli.pipelines.llava
import unitorch.cli.pipelines.mistral
import unitorch.cli.pipelines.sam



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.models.chinese_clip.modeling import (
    ChineseClipForPretrain,
    ChineseClipForClassification,
    ChineseClipForTextClassification,
    ChineseClipForImageClassification,
)
from unitorch.models.chinese_clip.processing import ChineseClipProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
import numpy as np
from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union


def tensor2vid(video: torch.Tensor) -> List[np.ndarray]:
    # prepare the final outputs
    i, f, c, h, w = video.shape
    images = video.permute(1, 3, 0, 4, 2).reshape(
        f, h, i * w, c
    )  # 1st (frames, h, batch_size, w, c) 2nd (frames, h, batch_size * w, c)
    images = images.unbind(dim=0)  # prepare a list of indvidual (consecutive frames)
    images = [
        (image.cpu().numpy() * 255).astype("uint8") for image in images
    ]  # f h w c
    return images



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import json
import logging
import torch
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from diffusers.utils import numpy_to_pil
from diffusers.models import ControlNetModel
from diffusers.pipelines import (
    StableDiffusionPipeline,
    StableDiffusionImg2ImgPipeline,
    StableDiffusionInpaintPipeline,
    StableDiffusionUpscalePipeline,
    StableDiffusionDepth2ImgPipeline,
    StableVideoDiffusionPipeline,
    StableDiffusionControlNetPipeline,
    StableDiffusionControlNetImg2ImgPipeline,
    StableDiffusionControlNetInpaintPipeline,
)
from unitorch import is_xformers_available
from unitorch.utils import is_remote_url
from unitorch.models.diffusers import StableForImage2VideoGeneration
from unitorch.models.diffusers import StableVideoProcessor
from unitorch.models.diffusers.modeling_stable import StableVideoDiffusionPipelineV2

from unitorch.utils import pop_value, nested_dict_value, tensor2vid
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
)
from unitorch.cli.models.diffusers import (
    pretrained_stable_infos,
    pretrained_stable_extensions_infos,
    load_weight,
)
from unitorch.cli.pipelines import Schedulers
from unitorch.cli.models.diffusion_utils import export_to_video


class StableForImage2VideoGenerationPipeline(StableForImage2VideoGeneration):
    def __init__(
        self,
        config_path: str,
        image_config_path: str,
        image_process_config_path: str,
        vae_config_path: str,
        scheduler_config_path: str,
        quant_config_path: Optional[str] = None,
        weight_path: Optional[Union[str, List[str]]] = None,
        state_dict: Optional[Dict[str, Any]] = None,
        device: Optional[Union[str, int]] = "cpu",
        enable_cpu_offload: Optional[bool] = False,
        enable_xformers: Optional[bool] = False,
    ):
        super().__init__(
            config_path=config_path,
            image_config_path=image_config_path,
            image_process_config_path=image_process_config_path,
            vae_config_path=vae_config_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
        )
        self.processor = StableVideoProcessor(
            vision_config_path=image_process_config_path,
            vae_config_path=vae_config_path,
        )
        self._device = "cpu" if device == "cpu" else int(device)

        self.from_pretrained(weight_path, state_dict=state_dict)
        self.eval()
        self.to(device=self._device)

        self._enable_cpu_offload = enable_cpu_offload
        self._enable_xformers = enable_xformers

    @classmethod
    @add_default_section_for_init("core/pipeline/stable/image2video")
    def from_core_configure(
        cls,
        config,
        pretrained_name: Optional[str] = "stable-v1.5",
        config_path: Optional[str] = None,
        image_config_path: Optional[str] = None,
        vae_config_path: Optional[str] = None,
        scheduler_config_path: Optional[str] = None,
        image_process_config_path: Optional[str] = None,
        quant_config_path: Optional[str] = None,
        pretrained_weight_path: Optional[str] = None,
        device: Optional[str] = "cpu",
        **kwargs,
    ):
        config.set_default_section("core/pipeline/stable/image2video")
        pretrained_name = config.getoption("pretrained_name", pretrained_name)
        pretrained_infos = nested_dict_value(pretrained_stable_infos, pretrained_name)

        config_path = config.getoption("config_path", config_path)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_infos, "unet", "config"),
        )
        config_path = cached_path(config_path)

        image_config_path = config.getoption("image_config_path", image_config_path)
        image_config_path = pop_value(
            image_config_path,
            nested_dict_value(pretrained_infos, "image", "config"),
        )
        image_config_path = cached_path(image_config_path)

        vae_config_path = config.getoption("vae_config_path", vae_config_path)
        vae_config_path = pop_value(
            vae_config_path,
            nested_dict_value(pretrained_infos, "vae", "config"),
        )
        vae_config_path = cached_path(vae_config_path)

        scheduler_config_path = config.getoption(
            "scheduler_config_path", scheduler_config_path
        )
        scheduler_config_path = pop_value(
            scheduler_config_path,
            nested_dict_value(pretrained_infos, "scheduler"),
        )
        scheduler_config_path = cached_path(scheduler_config_path)

        image_process_config_path = config.getoption(
            "image_process_config_path", image_process_config_path
        )
        image_process_config_path = pop_value(
            image_process_config_path,
            nested_dict_value(pretrained_infos, "image", "vision_config"),
        )
        image_process_config_path = cached_path(image_process_config_path)

        quant_config_path = config.getoption("quant_config_path", quant_config_path)
        if quant_config_path is not None:
            quant_config_path = cached_path(quant_config_path)

        weight_path = config.getoption("pretrained_weight_path", pretrained_weight_path)
        device = config.getoption("device", device)
        enable_cpu_offload = config.getoption("enable_cpu_offload", True)
        enable_xformers = config.getoption("enable_xformers", True)

        state_dict = None
        if weight_path is None and pretrained_infos is not None:
            state_dict = [
                load_weight(nested_dict_value(pretrained_infos, "unet", "weight")),
                load_weight(nested_dict_value(pretrained_infos, "image", "weight")),
                load_weight(nested_dict_value(pretrained_infos, "vae", "weight")),
            ]

        inst = cls(
            config_path=config_path,
            image_config_path=image_config_path,
            image_process_config_path=image_process_config_path,
            vae_config_path=vae_config_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            weight_path=weight_path,
            state_dict=state_dict,
            device=device,
            enable_cpu_offload=enable_cpu_offload,
            enable_xformers=enable_xformers,
        )
        return inst

    @torch.no_grad()
    @add_default_section_for_function("core/pipeline/stable/image2video")
    def __call__(
        self,
        image: Image.Image,
        height: Optional[int] = 576,
        width: Optional[int] = 1024,
        num_frames: Optional[int] = 30,
        num_fps: Optional[int] = 6,
        min_guidance_scale: Optional[float] = 1.0,
        max_guidance_scale: Optional[float] = 2.5,
        motion_bucket_id: Optional[int] = 127,
        decode_chunk_size: Optional[int] = 8,
        num_timesteps: Optional[int] = 50,
        seed: Optional[int] = 1123,
        scheduler: Optional[str] = None,
        lora_checkpoints: Optional[Union[str, List[str]]] = None,
        lora_weights: Optional[Union[float, List[float]]] = 1.0,
        lora_alphas: Optional[Union[float, List[float]]] = 32,
        lora_urls: Optional[Union[str, List[str]]] = None,
        lora_files: Optional[Union[str, List[str]]] = None,
    ):
        image = image.convert("RGB")
        inputs = self.processor.image2video_inputs(
            image,
            vae_image=image.resize((width, height)),
        )

        assert scheduler is None or scheduler in Schedulers
        if scheduler is not None:
            self.scheduler = Schedulers.get(scheduler).from_config(
                self.scheduler.config
            )
        self.scheduler.set_timesteps(num_inference_steps=num_timesteps)

        self.pipeline = StableVideoDiffusionPipelineV2(
            vae=self.vae,
            image_encoder=self.image,
            unet=self.unet,
            scheduler=self.scheduler,
            feature_extractor=self.image_processor,
        )
        self.pipeline.set_progress_bar_config(disable=True)
        self.seed = seed

        inputs = {k: v.unsqueeze(0) if v is not None else v for k, v in inputs.items()}
        inputs = {
            k: v.to(device=self._device) if v is not None else v
            for k, v in inputs.items()
        }
        if isinstance(lora_checkpoints, str):
            lora_checkpoints = [lora_checkpoints]
        if isinstance(lora_weights, float):
            lora_weights = [lora_weights]
        if isinstance(lora_alphas, float):
            lora_alphas = [lora_alphas]
        if isinstance(lora_urls, str):
            lora_urls = [lora_urls]
        if isinstance(lora_files, str):
            lora_files = [lora_files]

        if lora_checkpoints is not None:
            _lora_checkpoints = list(
                zip(
                    *[
                        (ckpt, weight, alpha)
                        for ckpt, weight, alpha in zip(
                            lora_checkpoints, lora_weights, lora_alphas
                        )
                        if ckpt is not None
                    ]
                )
            )
            if len(_lora_checkpoints) == 0:
                lora_checkpoints = None
            else:
                lora_checkpoints, lora_weights, lora_alphas = _lora_checkpoints
                lora_checkpoints = [
                    nested_dict_value(pretrained_stable_extensions_infos, ckpt)
                    for ckpt in lora_checkpoints
                ]
        if lora_urls is not None:
            _lora_urls = list(
                zip(
                    *[
                        (url, weight, alpha)
                        for url, weight, alpha in zip(
                            lora_urls, lora_weights, lora_alphas
                        )
                        if is_remote_url(url)
                    ]
                )
            )
            if len(_lora_urls) == 0:
                lora_urls = None
            else:
                lora_urls, lora_weights, lora_alphas = _lora_urls
        if lora_files is not None:
            _lora_files = list(
                zip(
                    *[
                        (file, weight, alpha)
                        for file, weight, alpha in zip(
                            lora_files, lora_weights, lora_alphas
                        )
                        if file is not None
                    ]
                )
            )
            if len(_lora_files) == 0:
                lora_files = None
            else:
                lora_files, lora_weights, lora_alphas = _lora_files

        lora_files = pop_value(
            lora_checkpoints,
            lora_urls,
            lora_files,
            check_none=False,
        )
        if lora_files is not None:
            self.load_lora_weights(
                lora_files, lora_weights=lora_weights, lora_alphas=lora_alphas
            )

        if self._enable_cpu_offload and self._device != "cpu":
            self.pipeline.enable_model_cpu_offload(self._device)
        else:
            self.to(device=self._device)

        if self._enable_xformers and self._device != "cpu":
            assert is_xformers_available(), "Please install xformers first."
            self.pipeline.enable_xformers_memory_efficient_attention()

        outputs = self.pipeline(
            image=inputs["pixel_values"],
            vae_image=inputs["vae_pixel_values"],
            height=height,
            width=width,
            num_frames=num_frames,
            min_guidance_scale=min_guidance_scale,
            max_guidance_scale=max_guidance_scale,
            fps=num_fps,
            motion_bucket_id=motion_bucket_id,
            decode_chunk_size=decode_chunk_size,
            generator=torch.Generator(device=self.pipeline.device).manual_seed(
                self.seed
            ),
            output_type="pt",
        )
        self.unload_lora_weights()

        frames = tensor2vid(outputs.frames)
        name = export_to_video(frames)
        return name



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from torch.cuda.amp import autocast
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.bart import BartForGeneration as _BartForGeneration
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_model,
)
from unitorch.cli.models import generation_model_decorator
from unitorch.cli.models import GenerationOutputs
from unitorch.cli.models.bart import pretrained_bart_infos


@register_model("core/model/generation/bart", generation_model_decorator)
class BartForGeneration(_BartForGeneration):
    """BART model for generation tasks."""

    def __init__(
        self,
        config_path: str,
        gradient_checkpointing: Optional[bool] = False,
    ):
        """
        Initialize the BartForGeneration model.

        Args:
            config_path (str): Path to the model configuration file.
            gradient_checkpointing (bool, optional): Whether to use gradient checkpointing for memory optimization.
        """
        super().__init__(
            config_path=config_path, gradient_checkpointing=gradient_checkpointing
        )

    @classmethod
    @add_default_section_for_init("core/model/generation/bart")
    def from_core_configure(cls, config, **kwargs):
        """
        Create an instance of BartForGeneration from core configuration.

        Args:
            config: The core configuration object.
            **kwargs: Additional keyword arguments.

        Returns:
            BartForGeneration: The initialized BartForGeneration instance.
        """
        config.set_default_section("core/model/generation/bart")
        pretrained_name = config.getoption("pretrained_name", "default-bart")
        config_path = config.getoption("config_path", None)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_bart_infos, pretrained_name, "config"),
        )

        config_path = cached_path(config_path)
        gradient_checkpointing = config.getoption("gradient_checkpointing", False)

        inst = cls(config_path, gradient_checkpointing)
        pretrained_weight_path = config.getoption("pretrained_weight_path", None)
        weight_path = pop_value(
            pretrained_weight_path,
            nested_dict_value(pretrained_bart_infos, pretrained_name, "weight"),
            check_none=False,
        )
        if weight_path is not None:
            inst.from_pretrained(weight_path)

        return inst

    @autocast()
    def forward(
        self,
        input_ids: torch.Tensor,
        attention_mask: torch.Tensor,
        decoder_input_ids: torch.Tensor,
        decoder_attention_mask: torch.Tensor,
    ):
        """
        Forward pass of the BartForGeneration model.

        Args:
            input_ids (torch.Tensor): Input IDs.
            attention_mask (torch.Tensor): Attention mask.
            decoder_input_ids (torch.Tensor): Decoder input IDs.
            decoder_attention_mask (torch.Tensor): Decoder attention mask.

        Returns:
            GenerationOutputs: The generated sequences and their scores.
        """
        outputs = super().forward(
            input_ids=input_ids,
            attention_mask=attention_mask,
            decoder_input_ids=decoder_input_ids,
            decoder_attention_mask=decoder_attention_mask,
        )
        return GenerationOutputs(sequences=outputs)

    @add_default_section_for_function("core/model/generation/bart")
    @torch.no_grad()
    @autocast()
    def generate(
        self,
        input_ids: torch.Tensor,
        num_beams: Optional[int] = 5,
        decoder_start_token_id: Optional[int] = 2,
        decoder_end_token_id: Optional[Union[int, List[int]]] = 2,
        num_return_sequences: Optional[int] = 1,
        min_gen_seq_length: Optional[int] = 0,
        max_gen_seq_length: Optional[int] = 48,
        repetition_penalty: Optional[float] = 1.0,
        no_repeat_ngram_size: Optional[int] = 0,
        early_stopping: Optional[bool] = True,
        length_penalty: Optional[float] = 1.0,
        num_beam_groups: Optional[int] = 1,
        diversity_penalty: Optional[float] = 0.0,
        do_sample: Optional[bool] = False,
        temperature: Optional[float] = 1.0,
        top_k: Optional[int] = 50,
        top_p: Optional[float] = 1.0,
    ):
        """
        Generate sequences using the BartForGeneration model.

        Args:
            input_ids (torch.Tensor): Input IDs.
            num_beams (int, optional): Number of beams for beam search.
            decoder_start_token_id (int, optional): ID of the decoder start token.
            decoder_end_token_id (int or List[int], optional): ID of the decoder end token.
            num_return_sequences (int, optional): Number of generated sequences to return.
            min_gen_seq_length (int, optional): Minimum length of generated sequences.
            max_gen_seq_length (int, optional): Maximum length of generated sequences.
            repetition_penalty (float, optional): Repetition penalty.
            no_repeat_ngram_size (int, optional): Size of n-grams to avoid repeating.
            early_stopping (bool, optional): Whether to stop generation early.
            length_penalty (float, optional): Length penalty for generated sequences.
            num_beam_groups (int, optional): Number of groups for diverse beam search.
            diversity_penalty (float, optional): Diversity penalty for diverse beam search.
            do_sample (bool, optional): Whether to use sampling for generation.
            temperature (float, optional): Sampling temperature.
            top_k (int, optional): Top-k sampling parameter.
            top_p (float, optional): Top-p sampling parameter.

        Returns:
            GenerationOutputs: The generated sequences and their scores.
        """
        outputs = super().generate(
            input_ids=input_ids,
            num_beams=num_beams,
            decoder_start_token_id=decoder_start_token_id,
            decoder_end_token_id=decoder_end_token_id,
            num_return_sequences=num_return_sequences,
            min_gen_seq_length=min_gen_seq_length,
            max_gen_seq_length=max_gen_seq_length,
            repetition_penalty=repetition_penalty,
            no_repeat_ngram_size=no_repeat_ngram_size,
            early_stopping=early_stopping,
            length_penalty=length_penalty,
            num_beam_groups=num_beam_groups,
            diversity_penalty=diversity_penalty,
            do_sample=do_sample,
            temperature=temperature,
            top_k=top_k,
            top_p=top_p,
        )

        return GenerationOutputs(
            sequences=outputs.sequences,
            sequences_scores=outputs.sequences_scores,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from functools import partial

import torch
from transformers import RobertaTokenizer
from unitorch.utils import pop_value, truncate_sequence_pair
from unitorch.models import GenericOutputs, HfTextClassificationProcessor


def get_roberta_tokenizer(vocab_path, merge_path):
    """
    Retrieves the RobertaTokenizer with the specified vocabulary and merge files.

    Args:
        vocab_path (str): The path to the vocabulary file.
        merge_path (str): The path to the merge file.

    Returns:
        RobertaTokenizer: The RobertaTokenizer instance.
    """
    assert os.path.exists(vocab_path) and os.path.exists(merge_path)
    tokenizer = RobertaTokenizer(vocab_path, merge_path)
    return tokenizer


class RobertaProcessor(HfTextClassificationProcessor):
    def __init__(
        self,
        vocab_path: str,
        merge_path: str,
        max_seq_length: Optional[int] = 128,
        source_type_id: Optional[int] = 0,
        target_type_id: Optional[int] = 0,
    ):
        """
        Initializes a RobertaProcessor for text classification tasks.

        Args:
            vocab_path (str): The path to the vocabulary file.
            merge_path (str): The path to the merge file.
            max_seq_length (int, optional): The maximum sequence length. Defaults to 128.
            source_type_id (int, optional): The ID for the source type. Defaults to 0.
            target_type_id (int, optional): The ID for the target type. Defaults to 0.
        """
        tokenizer = get_roberta_tokenizer(
            vocab_path,
            merge_path,
        )
        super().__init__(
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
            source_type_id=source_type_id,
            target_type_id=target_type_id,
            position_start_id=tokenizer.pad_token_id + 1,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import io
import torch
import gc
import gradio as gr
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.cli import CoreConfigureParser, GenericWebUI
from unitorch.cli import register_webui
from unitorch.cli.models.diffusers import (
    pretrained_stable_infos,
    pretrained_stable_extensions_infos,
)
from unitorch.cli.pipelines.stable import StableForImageResolutionPipeline
from unitorch.cli.pipelines.stable import controlnet_processes
from unitorch.cli.webuis import (
    supported_scheduler_names,
    matched_pretrained_names,
    create_element,
    create_accordion,
    create_row,
    create_column,
    create_group,
    create_tab,
    create_tabs,
    create_blocks,
    create_pretrain_layout,
    create_controlnet_layout,
    create_lora_layout,
    create_freeu_layout,
)
from unitorch.cli.webuis import SimpleWebUI


class StableImageResolutionWebUI(SimpleWebUI):
    pretrained_names = list(pretrained_stable_infos.keys())
    supported_pretrained_names = matched_pretrained_names(
        pretrained_names, ["^stable-v1.5-", "^stable-v2-", "^stable-v2.1"]
    )
    pretrained_extension_names = list(pretrained_stable_extensions_infos.keys())
    supported_lora_names = matched_pretrained_names(
        pretrained_extension_names,
        ["^stable-v1.5-lora-", "^stable-v2-lora-", "^stable-v2.1-lora-"],
    )
    supported_schedulers = supported_scheduler_names

    def __init__(self, config: CoreConfigureParser):
        self._pipe = None if not hasattr(self, "_pipe") else self._pipe
        self._status = "Stopped" if self._pipe is None else "Running"
        if len(self.supported_pretrained_names) == 0:
            raise ValueError("No supported pretrained models found.")
        self._name = self.supported_pretrained_names[0]

        # create elements
        pretrain_layout_group = create_pretrain_layout(
            self.supported_pretrained_names, self._name
        )
        name, status, start, stop, pretrain_layout = (
            pretrain_layout_group.name,
            pretrain_layout_group.status,
            pretrain_layout_group.start,
            pretrain_layout_group.stop,
            pretrain_layout_group.layout,
        )

        prompt = create_element(
            "text", "Input Prompt", lines=3, placeholder="Prompt", show_label=False
        )
        negative_prompt = create_element(
            "text",
            "Input Negative Prompt",
            lines=3,
            placeholder="Negative Prompt",
            show_label=False,
        )
        scheduler = create_element(
            "dropdown",
            "Sampling Method",
            values=self.supported_schedulers,
            default=self.supported_schedulers[0],
        )
        steps = create_element(
            "slider", "Diffusion Steps", min_value=1, max_value=100, step=1, default=25
        )
        image = create_element("image", "Input Image")

        guidance_scale = create_element(
            "slider", "Guidance Scale", min_value=0, max_value=10, step=0.1, default=7.5
        )
        noise_level = create_element(
            "slider", "Noise Level", min_value=0, max_value=50, step=0.1, default=20
        )

        seed = create_element(
            "slider", "Seed", min_value=0, max_value=9999, step=1, default=42
        )

        freeu_layout_group = create_freeu_layout()
        s1, s2, b1, b2, freeu_layout = (
            freeu_layout_group.s1,
            freeu_layout_group.s2,
            freeu_layout_group.b1,
            freeu_layout_group.b2,
            freeu_layout_group.layout,
        )

        ## extensions
        lora_layout_group = create_lora_layout(self.supported_lora_names, num_loras=5)
        loras = lora_layout_group.loras
        lora_layout = lora_layout_group.layout
        lora0, lora1, lora2, lora3, lora4 = loras
        (
            lora0_checkpoint,
            lora0_weight,
            lora0_alpha,
            lora0_url,
            lora0_file,
        ) = (
            lora0.checkpoint,
            lora0.weight,
            lora0.alpha,
            lora0.url,
            lora0.file,
        )
        (
            lora1_checkpoint,
            lora1_weight,
            lora1_alpha,
            lora1_url,
            lora1_file,
        ) = (
            lora1.checkpoint,
            lora1.weight,
            lora1.alpha,
            lora1.url,
            lora1.file,
        )
        (
            lora2_checkpoint,
            lora2_weight,
            lora2_alpha,
            lora2_url,
            lora2_file,
        ) = (
            lora2.checkpoint,
            lora2.weight,
            lora2.alpha,
            lora2.url,
            lora2.file,
        )
        (
            lora3_checkpoint,
            lora3_weight,
            lora3_alpha,
            lora3_url,
            lora3_file,
        ) = (
            lora3.checkpoint,
            lora3.weight,
            lora3.alpha,
            lora3.url,
            lora3.file,
        )
        (
            lora4_checkpoint,
            lora4_weight,
            lora4_alpha,
            lora4_url,
            lora4_file,
        ) = (
            lora4.checkpoint,
            lora4.weight,
            lora4.alpha,
            lora4.url,
            lora4.file,
        )
        generate = create_element("button", "Generate", variant="primary", scale=2)
        output_image = create_element("image", "Output Image")

        # create layouts
        top1 = create_column(pretrain_layout)
        top2 = create_row(
            create_column(prompt, negative_prompt, scale=4),
            create_column(generate),
        )
        left_generation = create_tab(
            create_row(image),
            create_row(scheduler, steps),
            create_row(guidance_scale),
            create_row(noise_level),
            create_row(seed),
            create_row(freeu_layout),
            name="Generation",
        )
        left_extension = create_tab(
            create_row(lora_layout),
            name="Extensions",
        )
        left_settings = create_tab(
            name="Settings",
        )

        left = create_tabs(left_generation, left_extension, left_settings)
        right = create_column(output_image)
        iface = create_blocks(top1, top2, create_row(left, right))

        # create events
        iface.__enter__()

        start.click(fn=self.start, inputs=[name], outputs=[status])
        stop.click(fn=self.stop, outputs=[status])

        generate.click(
            fn=self.serve,
            inputs=[
                prompt,
                image,
                negative_prompt,
                guidance_scale,
                noise_level,
                steps,
                seed,
                scheduler,
                s1,
                s2,
                b1,
                b2,
                lora0_checkpoint,
                lora0_weight,
                lora0_alpha,
                lora0_url,
                lora0_file,
                lora1_checkpoint,
                lora1_weight,
                lora1_alpha,
                lora1_url,
                lora1_file,
                lora2_checkpoint,
                lora2_weight,
                lora2_alpha,
                lora2_url,
                lora2_file,
                lora3_checkpoint,
                lora3_weight,
                lora3_alpha,
                lora3_url,
                lora3_file,
                lora4_checkpoint,
                lora4_weight,
                lora4_alpha,
                lora4_url,
                lora4_file,
            ],
            outputs=[output_image],
        )
        iface.load(
            fn=lambda: [gr.update(value=self._name), gr.update(value=self._status)],
            outputs=[name, status],
        )

        iface.__exit__()

        super().__init__(config, iname="Resolution", iface=iface)

    def start(self, pretrained_name, **kwargs):
        if self._status == "Running":
            self.stop()
        self._name = pretrained_name
        self._pipe = StableForImageResolutionPipeline.from_core_configure(
            self._config,
            pretrained_name=pretrained_name,
        )
        self._status = "Running"
        return self._status

    def stop(self, **kwargs):
        self._pipe.to("cpu")
        del self._pipe
        gc.collect()
        torch.cuda.empty_cache()
        self._pipe = None if not hasattr(self, "_pipe") else self._pipe
        self._status = "Stopped" if self._pipe is None else "Running"
        return self._status

    def serve(
        self,
        text: str,
        image: Image.Image,
        negative_text: Optional[str] = None,
        guidance_scale: Optional[float] = 7.5,
        noise_level: Optional[float] = 20,
        num_timesteps: Optional[int] = 25,
        seed: Optional[int] = 1123,
        scheduler: Optional[str] = None,
        freeu_s1: Optional[float] = 0.9,
        freeu_s2: Optional[float] = 0.2,
        freeu_b1: Optional[float] = 1.2,
        freeu_b2: Optional[float] = 1.4,
        lora0_checkpoint: Optional[str] = None,
        lora0_weight: Optional[float] = 1.0,
        lora0_alpha: Optional[float] = 32,
        lora0_url: Optional[str] = None,
        lora0_file: Optional[str] = None,
        lora1_checkpoint: Optional[str] = None,
        lora1_weight: Optional[float] = 1.0,
        lora1_alpha: Optional[float] = 32,
        lora1_url: Optional[str] = None,
        lora1_file: Optional[str] = None,
        lora2_checkpoint: Optional[str] = None,
        lora2_weight: Optional[float] = 1.0,
        lora2_alpha: Optional[float] = 32,
        lora2_url: Optional[str] = None,
        lora2_file: Optional[str] = None,
        lora3_checkpoint: Optional[str] = None,
        lora3_weight: Optional[float] = 1.0,
        lora3_alpha: Optional[float] = 32,
        lora3_url: Optional[str] = None,
        lora3_file: Optional[str] = None,
        lora4_checkpoint: Optional[str] = None,
        lora4_weight: Optional[float] = 1.0,
        lora4_alpha: Optional[float] = 32,
        lora4_url: Optional[str] = None,
        lora4_file: Optional[str] = None,
    ):
        assert self._pipe is not None
        image = self._pipe(
            text,
            image,
            negative_text,
            guidance_scale=guidance_scale,
            noise_level=noise_level,
            num_timesteps=num_timesteps,
            seed=seed,
            scheduler=scheduler,
            freeu_params=(freeu_s1, freeu_s2, freeu_b1, freeu_b2),
            lora_checkpoints=(
                lora0_checkpoint,
                lora1_checkpoint,
                lora2_checkpoint,
                lora3_checkpoint,
                lora4_checkpoint,
            ),
            lora_weights=(
                lora0_weight,
                lora1_weight,
                lora2_weight,
                lora3_weight,
                lora4_weight,
            ),
            lora_alphas=(
                lora0_alpha,
                lora1_alpha,
                lora2_alpha,
                lora3_alpha,
                lora4_alpha,
            ),
            lora_urls=(
                lora0_url,
                lora1_url,
                lora2_url,
                lora3_url,
                lora4_url,
            ),
            lora_files=(
                lora0_file,
                lora1_file,
                lora2_file,
                lora3_file,
                lora4_file,
            ),
        )
        return image



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import json
import random
import logging
import torch
import torch.nn.functional as F
from torch import nn
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from transformers.models.detr import DetrConfig
from transformers.models.detr.modeling_detr import (
    DetrModel,
    DetrMLPPredictionHead,
    DetrHungarianMatcher,
    DetrMaskHeadSmallConv,
    DetrMHAttentionMap,
    DetrLoss,
)
from unitorch.models import GenericModel, GenericOutputs
from unitorch.utils import image_list_to_tensor


class DetrForDetection(GenericModel):
    def __init__(
        self,
        config_path: str,
        num_classes: Optional[int] = None,
    ):
        super().__init__()
        config = DetrConfig.from_json_file(config_path)

        self.model = DetrModel(config)
        if num_classes is not None:
            config.num_labels = num_classes
        self.class_labels_classifier = nn.Linear(config.d_model, config.num_labels + 1)
        self.bbox_predictor = DetrMLPPredictionHead(
            input_dim=config.d_model,
            hidden_dim=config.d_model,
            output_dim=4,
            num_layers=3,
        )
        self.config = config
        self.init_weights()

        # modules for loss
        self.enable_auxiliary_loss = config.auxiliary_loss
        matcher = DetrHungarianMatcher(
            class_cost=config.class_cost,
            bbox_cost=config.bbox_cost,
            giou_cost=config.giou_cost,
        )
        losses = ["labels", "boxes", "cardinality"]
        self.criterion = DetrLoss(
            matcher=matcher,
            num_classes=config.num_labels,
            eos_coef=config.eos_coefficient,
            losses=losses,
        )
        self.criterion.to(self.device)
        self.weight_dict = {
            "loss_ce": 1,
            "loss_bbox": config.bbox_loss_coefficient,
            "loss_giou": config.giou_loss_coefficient,
        }
        if self.enable_auxiliary_loss:
            aux_weight_dict = {}
            for i in range(config.decoder_layers - 1):
                aux_weight_dict.update(
                    {k + f"_{i}": v for k, v in self.weight_dict.items()}
                )
            self.weight_dict.update(aux_weight_dict)

    @property
    def dtype(self):
        """
        `torch.dtype`: which dtype the parameters are (assuming that all the parameters are the same dtype).
        """

        return next(self.parameters()).dtype

    @property
    def device(self):
        """
        `torch.device`: The device on which the module is (assuming that all the module parameters are on the same device).
        """
        return next(self.parameters()).device

    def _set_aux_loss(self, outputs_class, outputs_coord):
        # this is a workaround to make torchscript happy, as torchscript
        # doesn't support dictionary with non-homogeneous values, such
        # as a dict having both a Tensor and a list.
        return [
            {"logits": a, "pred_boxes": b}
            for a, b in zip(outputs_class[:-1], outputs_coord[:-1])
        ]

    def forward(
        self,
        images: Union[List[torch.Tensor], torch.Tensor],
        bboxes: Union[List[torch.Tensor], torch.Tensor],
        classes: Union[List[torch.Tensor], torch.Tensor],
    ):
        if not isinstance(images, torch.Tensor):
            images = image_list_to_tensor(images)
        assert images.dim() == 4

        outputs = self.model(images.to(self.dtype))
        sequence_output = outputs[0]

        # class logits + predicted bounding boxes
        logits = self.class_labels_classifier(sequence_output)
        pred_boxes = self.bbox_predictor(sequence_output).sigmoid()
        outputs_loss = {}
        outputs_loss["logits"] = logits
        outputs_loss["pred_boxes"] = pred_boxes
        if self.enable_auxiliary_loss:
            intermediate = outputs[4]
            outputs_class = self.class_labels_classifier(intermediate)
            outputs_coord = self.bbox_predictor(intermediate).sigmoid()
            auxiliary_outputs = self._set_aux_loss(
                outputs_class,
                outputs_coord,
            )
            outputs_loss["auxiliary_outputs"] = auxiliary_outputs
        xyxy2xywh = lambda x: torch.stack(
            [
                (x[..., 0] + x[..., 2]) / 2,
                (x[..., 1] + x[..., 3]) / 2,
                x[..., 2] - x[..., 0],
                x[..., 3] - x[..., 1],
            ],
            -1,
        )
        bboxes = [xyxy2xywh(bbox) for bbox in bboxes]
        labels = [{"class_labels": c, "boxes": b} for b, c in zip(bboxes, classes)]
        loss_dict = self.criterion(outputs_loss, labels)
        loss = sum(
            loss_dict[k] * self.weight_dict[k]
            for k in loss_dict.keys()
            if k in self.weight_dict
        )
        return loss

    @torch.no_grad()
    def detect(
        self,
        images: Union[List[torch.Tensor], torch.Tensor],
        norm_bboxes: Optional[bool] = False,
        threshold: Optional[float] = 0.5,
    ):
        image_sizes = [(img.size(-2), img.size(-1)) for img in images]
        if not isinstance(images, torch.Tensor):
            images = image_list_to_tensor(images)
        assert images.dim() == 4

        outputs = self.model(images.to(self.dtype))
        sequence_output = outputs[0]

        # class logits + predicted bounding boxes
        logits = self.class_labels_classifier(sequence_output)
        pred_boxes = self.bbox_predictor(sequence_output).sigmoid()

        logits = logits.softmax(dim=-1)

        scores, classes = list(zip(*[p.max(-1) for p in logits]))
        xywh2xyxy = lambda x: torch.stack(
            [
                x[..., 0] - x[..., 2] / 2,
                x[..., 1] - x[..., 3] / 2,
                x[..., 0] + x[..., 2] / 2,
                x[..., 1] + x[..., 3] / 2,
            ],
            -1,
        )
        pred_boxes = [xywh2xyxy(bbox) for bbox in pred_boxes]
        if not norm_bboxes:
            sizes = image_sizes
            bboxes = [
                b * torch.tensor([s[1], s[0], s[1], s[0]]).to(b)
                for b, s in zip(pred_boxes, sizes)
            ]
        else:
            bboxes = pred_boxes

        bboxes, scores, classes = list(
            zip(
                *[
                    (
                        b[(c != self.config.num_labels) & (s > threshold)],
                        s[(c != self.config.num_labels) & (s > threshold)],
                        c[(c != self.config.num_labels) & (s > threshold)],
                    )
                    for b, s, c in zip(bboxes, scores, classes)
                ]
            )
        )

        outputs = dict(
            {
                "bboxes": list(bboxes),
                "scores": list(scores),
                "classes": list(classes),
            }
        )
        return GenericOutputs(outputs)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
import torch.nn as nn
from torch.optim import Adam, SGD
from transformers import AdamW
from unitorch.optims.lion import Lion

from unitorch.utils import is_bitsandbytes_available

if is_bitsandbytes_available():
    from bitsandbytes.optim import Adam8bit, AdamW8bit



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import unitorch.cli.models.peft.diffusers.modeling_controlnet_xl
import unitorch.cli.models.peft.diffusers.modeling_controlnet
import unitorch.cli.models.peft.diffusers.modeling_stable_xl
import unitorch.cli.models.peft.diffusers.modeling_stable
import unitorch.cli.models.peft.diffusers.modeling_adapter_xl
from unitorch.cli.models.peft.diffusers.modeling_controlnet_xl import (
    ControlNetXLLoraForText2ImageGeneration,
    ControlNetXLLoraForImage2ImageGeneration,
    ControlNetXLLoraForImageInpainting,
)
from unitorch.cli.models.peft.diffusers.modeling_controlnet import (
    ControlNetLoraForText2ImageGeneration,
    ControlNetLoraForImage2ImageGeneration,
    ControlNetLoraForImageInpainting,
)
from unitorch.cli.models.peft.diffusers.modeling_stable_xl import (
    StableXLLoraForText2ImageGeneration,
    StableXLLoraForImage2ImageGeneration,
    StableXLLoraForImageInpainting,
)
from unitorch.cli.models.peft.diffusers.modeling_stable import (
    StableLoraForText2ImageGeneration,
    StableLoraForImage2ImageGeneration,
    StableLoraForImageInpainting,
    StableLoraForImageResolution,
)
from unitorch.cli.models.peft.diffusers.modeling_adapter_xl import (
    StableXLAdapterLoraForText2ImageGeneration,
)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

# pretrained infos
pretrained_llava_infos = {
    "default-llava-v1.6": {
        "config": "https://huggingface.co/llava-hf/llava-v1.6-mistral-7b-hf/resolve/main/config.json",
        "vocab": "https://huggingface.co/llava-hf/llava-v1.6-mistral-7b-hf/resolve/main/tokenizer.model",
        "vision_config": "https://huggingface.co/llava-hf/llava-v1.6-mistral-7b-hf/resolve/main/preprocessor_config.json",
    },
    "llava-v1.6-mistral-7b-hf": {
        "config": "https://huggingface.co/llava-hf/llava-v1.6-mistral-7b-hf/resolve/main/config.json",
        "vocab": "https://huggingface.co/llava-hf/llava-v1.6-mistral-7b-hf/resolve/main/tokenizer.model",
        "vision_config": "https://huggingface.co/llava-hf/llava-v1.6-mistral-7b-hf/resolve/main/preprocessor_config.json",
        "weight": [
            f"https://huggingface.co/llava-hf/llava-v1.6-mistral-7b-hf/resolve/main/model-{str(i).rjust(5, '0')}-of-00004.safetensors"
            for i in range(1, 5)
        ],
    },
}

pretrained_llava_extensions_infos = {}

import unitorch.cli.models.llava.modeling
import unitorch.cli.models.llava.processing
from unitorch.cli.models.llava.modeling import (
    LlavaMistralClipForClassification,
    LlavaMistralClipForGeneration,
)
from unitorch.cli.models.llava.processing import LlavaMistralClipProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from torch.cuda.amp import autocast

from unitorch.models.peft.diffusers import (
    ControlNetXLLoraForText2ImageGeneration as _ControlNetXLLoraForText2ImageGeneration,
    ControlNetXLLoraForImage2ImageGeneration as _ControlNetXLLoraForImage2ImageGeneration,
    ControlNetXLLoraForImageInpainting as _ControlNetXLLoraForImageInpainting,
)
from unitorch.utils import pop_value, nested_dict_value
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_model,
)
from unitorch.cli.models import DiffusionOutputs, LossOutputs
from unitorch.cli.models import diffusion_model_decorator
from unitorch.cli.models.diffusers import (
    pretrained_stable_infos,
    pretrained_stable_extensions_infos,
    load_weight,
)


@register_model(
    "core/model/peft/lora/diffusers/text2image/controlnet_xl", diffusion_model_decorator
)
class ControlNetXLLoraForText2ImageGeneration(_ControlNetXLLoraForText2ImageGeneration):
    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        text2_config_path: str,
        vae_config_path: str,
        controlnet_configs_path: Union[str, List[str]],
        scheduler_config_path: str,
        quant_config_path: Optional[str] = None,
        image_size: Optional[int] = None,
        in_channels: Optional[int] = None,
        out_channels: Optional[int] = None,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        lora_r: Optional[int] = 16,
        seed: Optional[int] = 1123,
    ):
        super().__init__(
            config_path=config_path,
            text_config_path=text_config_path,
            text2_config_path=text2_config_path,
            vae_config_path=vae_config_path,
            controlnet_configs_path=controlnet_configs_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            image_size=image_size,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            lora_r=lora_r,
            seed=seed,
        )

    @classmethod
    @add_default_section_for_init(
        "core/model/peft/lora/diffusers/text2image/controlnet_xl"
    )
    def from_core_configure(cls, config, **kwargs):
        config.set_default_section(
            "core/model/peft/lora/diffusers/text2image/controlnet_xl"
        )
        pretrained_name = config.getoption("pretrained_name", "stable-xl-base")
        pretrained_infos = nested_dict_value(pretrained_stable_infos, pretrained_name)

        pretrained_controlnet_name = config.getoption(
            "pretrained_controlnet_name", "stable-xl-controlnet-canny"
        )
        pretrained_controlnet_infos = nested_dict_value(
            pretrained_stable_extensions_infos, pretrained_controlnet_name
        )

        config_path = config.getoption("config_path", None)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_infos, "unet", "config"),
        )
        config_path = cached_path(config_path)

        text_config_path = config.getoption("text_config_path", None)
        text_config_path = pop_value(
            text_config_path,
            nested_dict_value(pretrained_infos, "text", "config"),
        )
        text_config_path = cached_path(text_config_path)

        text2_config_path = config.getoption("text2_config_path", None)
        text2_config_path = pop_value(
            text2_config_path,
            nested_dict_value(pretrained_infos, "text2", "config"),
        )
        text2_config_path = cached_path(text2_config_path)

        vae_config_path = config.getoption("vae_config_path", None)
        vae_config_path = pop_value(
            vae_config_path,
            nested_dict_value(pretrained_infos, "vae", "config"),
        )
        vae_config_path = cached_path(vae_config_path)

        controlnet_configs_path = config.getoption("controlnet_configs_path", None)
        controlnet_configs_path = pop_value(
            controlnet_configs_path,
            nested_dict_value(pretrained_controlnet_infos, "controlnet", "config"),
        )
        controlnet_configs_path = cached_path(controlnet_configs_path)

        scheduler_config_path = config.getoption("scheduler_config_path", None)
        scheduler_config_path = pop_value(
            scheduler_config_path,
            nested_dict_value(pretrained_infos, "scheduler"),
        )
        scheduler_config_path = cached_path(scheduler_config_path)

        quant_config_path = config.getoption("quant_config_path", None)
        if quant_config_path is not None:
            quant_config_path = cached_path(quant_config_path)

        image_size = config.getoption("image_size", None)
        in_channels = config.getoption("in_channels", None)
        out_channels = config.getoption("out_channels", None)
        num_train_timesteps = config.getoption("num_train_timesteps", 1000)
        num_infer_timesteps = config.getoption("num_infer_timesteps", 50)
        lora_r = config.getoption("lora_r", 16)
        seed = config.getoption("seed", 1123)

        inst = cls(
            config_path=config_path,
            text_config_path=text_config_path,
            text2_config_path=text2_config_path,
            vae_config_path=vae_config_path,
            controlnet_configs_path=controlnet_configs_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            image_size=image_size,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            lora_r=lora_r,
            seed=seed,
        )

        weight_path = config.getoption("pretrained_weight_path", None)

        state_dict = None
        if weight_path is None and pretrained_infos is not None:
            state_dict = [
                load_weight(
                    nested_dict_value(pretrained_infos, "unet", "weight"),
                    prefix_keys={"": "unet."},
                    replace_keys={
                        "to_k.": "to_k.base_layer.",
                        "to_q.": "to_q.base_layer.",
                        "to_v.": "to_v.base_layer.",
                        "to_out.0.": "to_out.0.base_layer.",
                    },
                ),
                load_weight(
                    nested_dict_value(pretrained_infos, "text", "weight"),
                    prefix_keys={"": "text."},
                ),
                load_weight(
                    nested_dict_value(pretrained_infos, "text2", "weight"),
                    prefix_keys={"": "text2."},
                ),
                load_weight(
                    nested_dict_value(pretrained_infos, "vae", "weight"),
                    prefix_keys={"": "vae."},
                ),
                load_weight(
                    nested_dict_value(
                        pretrained_controlnet_infos, "controlnet", "weight"
                    ),
                    prefix_keys={"": "controlnet."},
                ),
            ]
        elif weight_path is not None:
            state_dict = load_weight(weight_path)

        pretrained_lora_weight_path = config.getoption(
            "pretrained_lora_weight_path", None
        )
        if pretrained_lora_weight_path is not None:
            lora_state_dict = load_weight(pretrained_lora_weight_path)
            if state_dict is not None:
                state_dict.append(lora_state_dict)
            else:
                state_dict = lora_state_dict

        if state_dict is not None:
            inst.from_pretrained(state_dict=state_dict)

        return inst

    # @autocast()
    def forward(
        self,
        input_ids: torch.Tensor,
        input2_ids: torch.Tensor,
        add_time_ids: torch.Tensor,
        pixel_values: torch.Tensor,
        condition_pixel_values: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        attention2_mask: Optional[torch.Tensor] = None,
    ):
        loss = super().forward(
            input_ids=input_ids,
            input2_ids=input2_ids,
            add_time_ids=add_time_ids,
            pixel_values=pixel_values,
            condition_pixel_values=condition_pixel_values,
            attention_mask=attention_mask,
            attention2_mask=attention2_mask,
        )
        return LossOutputs(loss=loss)

    @add_default_section_for_function(
        "core/model/peft/lora/diffusers/text2image/controlnet_xl"
    )
    # @autocast()
    def generate(
        self,
        input_ids: torch.Tensor,
        negative_input_ids: torch.Tensor,
        condition_pixel_values: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        negative_attention_mask: Optional[torch.Tensor] = None,
        height: Optional[int] = 1024,
        width: Optional[int] = 1024,
        guidance_scale: Optional[float] = 7.5,
        controlnet_conditioning_scale: Optional[float] = 0.5,
    ):
        outputs = super().generate(
            input_ids=input_ids,
            negative_input_ids=negative_input_ids,
            condition_pixel_values=condition_pixel_values,
            attention_mask=attention_mask,
            negative_attention_mask=negative_attention_mask,
            height=height,
            width=width,
            guidance_scale=guidance_scale,
            controlnet_conditioning_scale=controlnet_conditioning_scale,
        )

        return DiffusionOutputs(outputs=outputs.images)


@register_model(
    "core/model/peft/lora/diffusers/image2image/controlnet_xl",
    diffusion_model_decorator,
)
class ControlNetXLLoraForImage2ImageGeneration(
    _ControlNetXLLoraForImage2ImageGeneration
):
    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        text2_config_path: str,
        vae_config_path: str,
        controlnet_configs_path: Union[str, List[str]],
        scheduler_config_path: str,
        quant_config_path: Optional[str] = None,
        image_size: Optional[int] = None,
        in_channels: Optional[int] = None,
        out_channels: Optional[int] = None,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        lora_r: Optional[int] = 16,
        seed: Optional[int] = 1123,
    ):
        super().__init__(
            config_path=config_path,
            text_config_path=text_config_path,
            text2_config_path=text2_config_path,
            vae_config_path=vae_config_path,
            controlnet_configs_path=controlnet_configs_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            image_size=image_size,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            lora_r=lora_r,
            seed=seed,
        )

    @classmethod
    @add_default_section_for_init(
        "core/model/peft/lora/diffusers/image2image/controlnet_xl"
    )
    def from_core_configure(cls, config, **kwargs):
        config.set_default_section(
            "core/model/peft/lora/diffusers/image2image/controlnet_xl"
        )
        pretrained_name = config.getoption("pretrained_name", "stable-xl-base")
        pretrained_infos = nested_dict_value(pretrained_stable_infos, pretrained_name)

        pretrained_controlnet_name = config.getoption(
            "pretrained_controlnet_name", "stable-xl-controlnet-canny"
        )
        pretrained_controlnet_infos = nested_dict_value(
            pretrained_stable_extensions_infos, pretrained_controlnet_name
        )

        config_path = config.getoption("config_path", None)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_infos, "unet", "config"),
        )
        config_path = cached_path(config_path)

        text_config_path = config.getoption("text_config_path", None)
        text_config_path = pop_value(
            text_config_path,
            nested_dict_value(pretrained_infos, "text", "config"),
        )
        text_config_path = cached_path(text_config_path)

        text2_config_path = config.getoption("text2_config_path", None)
        text2_config_path = pop_value(
            text2_config_path,
            nested_dict_value(pretrained_infos, "text2", "config"),
        )
        text2_config_path = cached_path(text2_config_path)

        vae_config_path = config.getoption("vae_config_path", None)
        vae_config_path = pop_value(
            vae_config_path,
            nested_dict_value(pretrained_infos, "vae", "config"),
        )
        vae_config_path = cached_path(vae_config_path)

        controlnet_configs_path = config.getoption("controlnet_configs_path", None)
        controlnet_configs_path = pop_value(
            controlnet_configs_path,
            nested_dict_value(pretrained_controlnet_infos, "controlnet", "config"),
        )
        controlnet_configs_path = cached_path(controlnet_configs_path)

        scheduler_config_path = config.getoption("scheduler_config_path", None)
        scheduler_config_path = pop_value(
            scheduler_config_path,
            nested_dict_value(pretrained_infos, "scheduler"),
        )
        scheduler_config_path = cached_path(scheduler_config_path)

        quant_config_path = config.getoption("quant_config_path", None)
        if quant_config_path is not None:
            quant_config_path = cached_path(quant_config_path)

        image_size = config.getoption("image_size", None)
        in_channels = config.getoption("in_channels", None)
        out_channels = config.getoption("out_channels", None)
        num_train_timesteps = config.getoption("num_train_timesteps", 1000)
        num_infer_timesteps = config.getoption("num_infer_timesteps", 50)
        lora_r = config.getoption("lora_r", 16)
        seed = config.getoption("seed", 1123)

        inst = cls(
            config_path=config_path,
            text_config_path=text_config_path,
            text2_config_path=text2_config_path,
            vae_config_path=vae_config_path,
            controlnet_configs_path=controlnet_configs_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            image_size=image_size,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            lora_r=lora_r,
            seed=seed,
        )

        weight_path = config.getoption("pretrained_weight_path", None)

        state_dict = None
        if weight_path is None and pretrained_infos is not None:
            state_dict = [
                load_weight(
                    nested_dict_value(pretrained_infos, "unet", "weight"),
                    prefix_keys={"": "unet."},
                    replace_keys={
                        "to_k.": "to_k.base_layer.",
                        "to_q.": "to_q.base_layer.",
                        "to_v.": "to_v.base_layer.",
                        "to_out.0.": "to_out.0.base_layer.",
                    },
                ),
                load_weight(
                    nested_dict_value(pretrained_infos, "text", "weight"),
                    prefix_keys={"": "text."},
                ),
                load_weight(
                    nested_dict_value(pretrained_infos, "text2", "weight"),
                    prefix_keys={"": "text2."},
                ),
                load_weight(
                    nested_dict_value(pretrained_infos, "vae", "weight"),
                    prefix_keys={"": "vae."},
                ),
                load_weight(
                    nested_dict_value(
                        pretrained_controlnet_infos, "controlnet", "weight"
                    ),
                    prefix_keys={"": "controlnet."},
                ),
            ]
        elif weight_path is not None:
            state_dict = load_weight(weight_path)

        pretrained_lora_weight_path = config.getoption(
            "pretrained_lora_weight_path", None
        )
        if pretrained_lora_weight_path is not None:
            lora_state_dict = load_weight(pretrained_lora_weight_path)
            if state_dict is not None:
                state_dict.append(lora_state_dict)
            else:
                state_dict = lora_state_dict

        if state_dict is not None:
            inst.from_pretrained(state_dict=state_dict)

        return inst

    # @autocast()
    def forward(
        self,
    ):
        raise NotImplementedError

    @add_default_section_for_function(
        "core/model/peft/lora/diffusers/image2image/controlnet_xl"
    )
    # @autocast()
    def generate(
        self,
        input_ids: torch.Tensor,
        negative_input_ids: torch.Tensor,
        pixel_values: torch.Tensor,
        condition_pixel_values: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        negative_attention_mask: Optional[torch.Tensor] = None,
        strength: Optional[float] = 0.8,
        guidance_scale: Optional[float] = 7.5,
        controlnet_conditioning_scale: Optional[float] = 0.5,
    ):
        outputs = super().generate(
            input_ids=input_ids,
            negative_input_ids=negative_input_ids,
            pixel_values=pixel_values,
            condition_pixel_values=condition_pixel_values,
            attention_mask=attention_mask,
            negative_attention_mask=negative_attention_mask,
            strength=strength,
            guidance_scale=guidance_scale,
            controlnet_conditioning_scale=controlnet_conditioning_scale,
        )
        return DiffusionOutputs(outputs=outputs.images)


@register_model(
    "core/model/peft/lora/diffusers/inpainting/controlnet_xl", diffusion_model_decorator
)
class ControlNetXLLoraForImageInpainting(_ControlNetXLLoraForImageInpainting):
    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        text2_config_path: str,
        vae_config_path: str,
        controlnet_configs_path: Union[str, List[str]],
        scheduler_config_path: str,
        quant_config_path: Optional[str] = None,
        image_size: Optional[int] = None,
        in_channels: Optional[int] = None,
        out_channels: Optional[int] = None,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        lora_r: Optional[int] = 16,
        seed: Optional[int] = 1123,
    ):
        super().__init__(
            config_path=config_path,
            text_config_path=text_config_path,
            text2_config_path=text2_config_path,
            vae_config_path=vae_config_path,
            controlnet_configs_path=controlnet_configs_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            image_size=image_size,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            lora_r=lora_r,
            seed=seed,
        )

    @classmethod
    @add_default_section_for_init(
        "core/model/peft/lora/diffusers/inpainting/controlnet_xl"
    )
    def from_core_configure(cls, config, **kwargs):
        config.set_default_section(
            "core/model/peft/lora/diffusers/inpainting/controlnet_xl"
        )
        pretrained_name = config.getoption("pretrained_name", "stable-xl-base")
        pretrained_infos = nested_dict_value(pretrained_stable_infos, pretrained_name)

        pretrained_controlnet_name = config.getoption(
            "pretrained_controlnet_name", "stable-xl-controlnet-canny"
        )
        pretrained_controlnet_infos = nested_dict_value(
            pretrained_stable_extensions_infos, pretrained_controlnet_name
        )

        config_path = config.getoption("config_path", None)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_infos, "unet", "config"),
        )
        config_path = cached_path(config_path)

        text_config_path = config.getoption("text_config_path", None)
        text_config_path = pop_value(
            text_config_path,
            nested_dict_value(pretrained_infos, "text", "config"),
        )
        text_config_path = cached_path(text_config_path)

        text2_config_path = config.getoption("text2_config_path", None)
        text2_config_path = pop_value(
            text2_config_path,
            nested_dict_value(pretrained_infos, "text2", "config"),
        )
        text2_config_path = cached_path(text2_config_path)

        vae_config_path = config.getoption("vae_config_path", None)
        vae_config_path = pop_value(
            vae_config_path,
            nested_dict_value(pretrained_infos, "vae", "config"),
        )
        vae_config_path = cached_path(vae_config_path)

        controlnet_configs_path = config.getoption("controlnet_configs_path", None)
        controlnet_configs_path = pop_value(
            controlnet_configs_path,
            nested_dict_value(pretrained_controlnet_infos, "controlnet", "config"),
        )
        controlnet_configs_path = cached_path(controlnet_configs_path)

        scheduler_config_path = config.getoption("scheduler_config_path", None)
        scheduler_config_path = pop_value(
            scheduler_config_path,
            nested_dict_value(pretrained_infos, "scheduler"),
        )
        scheduler_config_path = cached_path(scheduler_config_path)

        quant_config_path = config.getoption("quant_config_path", None)
        if quant_config_path is not None:
            quant_config_path = cached_path(quant_config_path)

        image_size = config.getoption("image_size", None)
        in_channels = config.getoption("in_channels", None)
        out_channels = config.getoption("out_channels", None)
        num_train_timesteps = config.getoption("num_train_timesteps", 1000)
        num_infer_timesteps = config.getoption("num_infer_timesteps", 50)
        lora_r = config.getoption("lora_r", 16)
        seed = config.getoption("seed", 1123)

        inst = cls(
            config_path=config_path,
            text_config_path=text_config_path,
            text2_config_path=text2_config_path,
            vae_config_path=vae_config_path,
            controlnet_configs_path=controlnet_configs_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            image_size=image_size,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            lora_r=lora_r,
            seed=seed,
        )

        weight_path = config.getoption("pretrained_weight_path", None)

        state_dict = None
        if weight_path is None and pretrained_infos is not None:
            state_dict = [
                load_weight(
                    nested_dict_value(pretrained_infos, "unet", "weight"),
                    prefix_keys={"": "unet."},
                    replace_keys={
                        "to_k.": "to_k.base_layer.",
                        "to_q.": "to_q.base_layer.",
                        "to_v.": "to_v.base_layer.",
                        "to_out.0.": "to_out.0.base_layer.",
                    },
                ),
                load_weight(
                    nested_dict_value(pretrained_infos, "text", "weight"),
                    prefix_keys={"": "text."},
                ),
                load_weight(
                    nested_dict_value(pretrained_infos, "text2", "weight"),
                    prefix_keys={"": "text2."},
                ),
                load_weight(
                    nested_dict_value(pretrained_infos, "vae", "weight"),
                    prefix_keys={"": "vae."},
                ),
                load_weight(
                    nested_dict_value(
                        pretrained_controlnet_infos, "controlnet", "weight"
                    ),
                    prefix_keys={"": "controlnet."},
                ),
            ]
        elif weight_path is not None:
            state_dict = load_weight(weight_path)

        pretrained_lora_weight_path = config.getoption(
            "pretrained_lora_weight_path", None
        )
        if pretrained_lora_weight_path is not None:
            lora_state_dict = load_weight(pretrained_lora_weight_path)
            if state_dict is not None:
                state_dict.append(lora_state_dict)
            else:
                state_dict = lora_state_dict

        if state_dict is not None:
            inst.from_pretrained(state_dict=state_dict)

        return inst

    # @autocast()
    def forward(
        self,
    ):
        raise NotImplementedError

    @add_default_section_for_function(
        "core/model/peft/lora/diffusers/inpainting/controlnet_xl"
    )
    # @autocast()
    def generate(
        self,
        input_ids: torch.Tensor,
        negative_input_ids: torch.Tensor,
        pixel_values: torch.Tensor,
        pixel_masks: torch.Tensor,
        condition_pixel_values: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        negative_attention_mask: Optional[torch.Tensor] = None,
        strength: Optional[float] = 0.8,
        guidance_scale: Optional[float] = 7.5,
        controlnet_conditioning_scale: Optional[float] = 0.5,
    ):
        outputs = super().generate(
            input_ids=input_ids,
            negative_input_ids=negative_input_ids,
            pixel_values=pixel_values,
            pixel_masks=pixel_masks,
            condition_pixel_values=condition_pixel_values,
            attention_mask=attention_mask,
            negative_attention_mask=negative_attention_mask,
            strength=strength,
            guidance_scale=guidance_scale,
            controlnet_conditioning_scale=controlnet_conditioning_scale,
        )
        return DiffusionOutputs(outputs=outputs.images)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import re
import torch
import numpy as np
from PIL import Image, ImageDraw
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.dpt import (
    DPTForDepthEstimation as _DPTForDepthEstimation,
    DPTProcessor,
)
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
)
from unitorch.cli.models.dpt import pretrained_dpt_infos


class DPTForDepthEstimationPipeline(_DPTForDepthEstimation):
    def __init__(
        self,
        config_path: str,
        vision_config_path: str,
        weight_path: Optional[Union[str, List[str]]] = None,
        state_dict: Optional[Dict[str, Any]] = None,
        device: Optional[Union[str, int]] = "cpu",
    ):
        super().__init__(
            config_path=config_path,
        )
        self.processor = DPTProcessor(
            vision_config_path=vision_config_path,
        )
        self._device = "cpu" if device == "cpu" else int(device)

        self.from_pretrained(weight_path, state_dict=state_dict)
        self.to(device=self._device)
        self.eval()

    @classmethod
    @add_default_section_for_init("core/pipeline/dpt")
    def from_core_configure(
        cls,
        config,
        pretrained_name: Optional[str] = "default-dpt",
        config_path: Optional[str] = None,
        vision_config_path: Optional[str] = None,
        pretrained_weight_path: Optional[str] = None,
        device: Optional[str] = "cpu",
        **kwargs,
    ):
        config.set_default_section("core/pipeline/dpt")

        pretrained_name = config.getoption("pretrained_name", pretrained_name)
        config_path = config.getoption("config_path", config_path)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_dpt_infos, pretrained_name, "config"),
        )
        config_path = cached_path(config_path)

        vision_config_path = config.getoption("vision_config_path", vision_config_path)
        vision_config_path = pop_value(
            vision_config_path,
            nested_dict_value(pretrained_dpt_infos, pretrained_name, "vision_config"),
        )
        vision_config_path = cached_path(vision_config_path)

        device = config.getoption("device", device)
        pretrained_weight_path = config.getoption(
            "pretrained_weight_path", pretrained_weight_path
        )
        weight_path = pop_value(
            pretrained_weight_path,
            nested_dict_value(pretrained_dpt_infos, pretrained_name, "weight"),
            check_none=False,
        )

        inst = cls(
            config_path=config_path,
            vision_config_path=vision_config_path,
            weight_path=weight_path,
            device=device,
        )

        return inst

    @torch.no_grad()
    @add_default_section_for_function("core/pipeline/dpt")
    def __call__(
        self,
        image: Union[Image.Image, str],
    ):
        inputs = self.processor.classification(image)
        pixel_values = inputs.pixel_values.unsqueeze(0).to(self._device)
        outputs = self.forward(
            pixel_values,
        )
        masks = outputs[0].cpu().numpy().squeeze(0)
        result_image = Image.fromarray((masks * 255) / np.max(masks))
        result_image = result_image.resize(image.size)

        return result_image



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from functools import partial
from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union


def rpartial(
    func,
    *args,
    **kwargs,
):
    """
    Create a new function that partially applies the given arguments to the provided callable.

    Args:
        func: The callable object or function to be partially applied.
        *args: Positional arguments to be partially applied.
        **kwargs: Keyword arguments to be partially applied.

    Returns:
        A new function that, when called, will invoke `func` with the original and partially applied arguments.
    """
    return lambda *a, **kw: func(*(args + a), **{**kwargs, **kw})


def pop_value(
    *args,
    msg: Optional[str] = "default error msg",
    first: Optional[bool] = True,
    last: Optional[bool] = False,
    check_none: Optional[bool] = True,
) -> Any:
    """
    Returns the first non-None value from the list of arguments.

    Args:
        *args: A list of Python values.
        msg (str, optional): Error message to be raised if no non-None value is found (default: "default error msg").
        first (bool, optional): If True, returns the first non-None value. If False, returns the last non-None value (default: True).
        last (bool, optional): If True, returns the last non-None value. If False, returns the first non-None value (default: False).
        check_none (bool, optional): If True, raises a ValueError if no non-None value is found. If False, returns None (default: True).

    Raises:
        ValueError: If check_none is True and no non-None value is found.

    Returns:
        The first non-None value if found, None otherwise.
    """
    assert first is True or last is True

    if first:
        for arg in args:
            if arg is not None:
                return arg
    else:
        for arg in reversed(args):
            if arg is not None:
                return arg

    if check_none:
        raise ValueError(f"{msg} Can't find non-None value")

    return None


def truncate_sequence_pair(
    tokens: List[str], tokens_pair: List[str], max_length: int
) -> None:
    while True:
        total_length = len(tokens) + len(tokens_pair)
        if total_length <= max_length:
            break
        if len(tokens) > len(tokens_pair):
            tokens.pop()
        else:
            tokens_pair.pop()


nested_dict_value = (
    lambda a, b, *c: (
        nested_dict_value(a[b], *c) if isinstance(a[b], dict) and len(c) > 0 else a[b]
    )
    if b in a
    else None
)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import torch
import numpy as np
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from torchvision.transforms import Resize, CenterCrop, ToTensor, Normalize, Compose
from transformers import CLIPImageProcessor, CLIPTokenizer
from transformers.image_utils import to_numpy_array, ChannelDimension
from transformers.image_transforms import to_channel_dimension_format
from unitorch.utils import pop_value
from unitorch.models import (
    HfImageClassificationProcessor,
    HfTextClassificationProcessor,
    GenericOutputs,
)


class ClipProcessor(HfImageClassificationProcessor, HfTextClassificationProcessor):
    def __init__(
        self,
        vocab_path: Optional[str] = None,
        merge_path: Optional[str] = None,
        vision_config_path: Optional[str] = None,
        max_seq_length: Optional[int] = 128,
        position_start_id: Optional[int] = 0,
    ):
        """
        Initializes the ClipProcessor.

        Args:
            vocab_path (str): The path to the vocabulary file.
            merge_path (str): The path to the merge file.
            vision_config_path (str): The path to the vision configuration file.
            max_seq_length (int, optional): The maximum sequence length for text inputs. Defaults to 128.
            position_start_id (int, optional): The starting position ID for positional embeddings. Defaults to 0.
        """
        if vision_config_path is not None:
            vision_processor = CLIPImageProcessor.from_json_file(vision_config_path)
        else:
            vision_processor = CLIPImageProcessor.from_pretrained(
                "openai/clip-vit-base-patch32"
            )
        HfImageClassificationProcessor.__init__(
            self,
            vision_processor=vision_processor,
        )

        if vocab_path is not None and merge_path is not None:
            tokenizer = CLIPTokenizer(
                vocab_file=vocab_path,
                merges_file=merge_path,
            )
        else:
            tokenizer = CLIPTokenizer.from_pretrained("openai/clip-vit-base-patch32")
        tokenizer.cls_token = tokenizer.bos_token
        tokenizer.sep_token = tokenizer.eos_token
        HfTextClassificationProcessor.__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
            source_type_id=0,
            target_type_id=0,
            position_start_id=position_start_id,
        )

    def text_classification(
        self,
        text: str,
        max_seq_length: Optional[int] = None,
    ):
        """
        Performs text classification.

        Args:
            text (str): The input text.
            max_seq_length (int, optional): The maximum sequence length for text inputs. Defaults to None.

        Returns:
            GenericOutputs: An object containing the processed inputs.
        """
        outputs = HfTextClassificationProcessor.classification(
            self,
            text=text,
            max_seq_length=max_seq_length,
        )
        return GenericOutputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            position_ids=outputs.position_ids,
        )

    def image_classification(
        self,
        image: Union[Image.Image, str],
    ):
        """
        Performs image classification.

        Args:
            image (PIL.Image.Image): The input image.

        Returns:
            GenericOutputs: An object containing the processed inputs.
        """
        outputs = HfImageClassificationProcessor.classification(
            self,
            image=image,
        )

        return GenericOutputs(
            pixel_values=outputs.pixel_values,
        )

    def classification(
        self,
        text: str,
        image: Union[Image.Image, str],
        max_seq_length: Optional[int] = None,
    ):
        """
        Performs classification using text and image inputs.

        Args:
            text (str): The input text.
            image (PIL.Image.Image): The input image.
            max_seq_length (int, optional): The maximum sequence length for text inputs. Defaults to None.

        Returns:
            GenericOutputs: An object containing the processed inputs.
        """
        text_outputs = self.text_classification(
            text=text,
            max_seq_length=max_seq_length,
        )
        pixel_outputs = self.image_classification(
            image=image,
        )

        return GenericOutputs(
            input_ids=text_outputs.input_ids,
            attention_mask=text_outputs.attention_mask,
            position_ids=text_outputs.position_ids,
            pixel_values=pixel_outputs.pixel_values,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import inspect
import logging
from unitorch.cli import get_global_config

__global_config__ = get_global_config()


def add_default_section_for_init(section, default_params=dict()):
    def default_init_func(cls, config, **kwargs):
        sign = inspect.signature(cls)
        params = sign.parameters
        for k, v in params.items():
            dvalue = v.default
            if kwargs.get(k) is not None:
                continue
            kwargs[k] = config.getdefault(section, k, dvalue)
        for k, v in default_params.items():
            kwargs[k] = config.getdefault(section, k, v)
        obj = cls(**kwargs)
        setattr(obj, "__unitorch_setting__", config)
        return obj

    def add_func(init_func):
        def _init_func(cls, config, **kwargs):
            ret = init_func(cls, config, **kwargs)
            if isinstance(ret, cls):
                setattr(ret, "__unitorch_setting__", config)
                return ret
            assert isinstance(ret, dict) or ret is None
            if ret is not None:
                kwargs.update(ret)
            ret = default_init_func(cls, config, **kwargs)
            return ret

        return _init_func

    return add_func


def add_default_section_for_function(
    section,
    default_params=dict(),
):
    def get_func_params(
        func,
        config,
        args,
        kwargs,
    ):
        sign = inspect.signature(func)
        params = sign.parameters
        for k, v in default_params.items():
            if k in kwargs:
                continue
            kwargs[k] = config.getdefault(section, k, v)

        for i, (k, v) in enumerate(params.items()):
            if k == "self" or k in kwargs:
                continue
            dvalue = args[i] if i < len(args) else v.default
            kwargs[k] = config.getdefault(section, k, dvalue)
        return kwargs

    def add_func(func):
        def _new_func(*args, **kwargs):
            if len(args) > 0 and hasattr(args[0], "__unitorch_setting__"):
                kwargs = get_func_params(
                    func,
                    args[0].__unitorch_setting__,
                    args,
                    kwargs,
                )
                ret = func(args[0], **kwargs)
            else:
                kwargs = get_func_params(
                    func,
                    __global_config__,
                    args,
                    kwargs,
                )
                ret = func(args[0], **kwargs)
            return ret

        return _new_func

    return add_func


# import gradio as gr
# def save_and_load_latest_state(init_func):
#     dtypes = [
#         gr.Dropdown,
#         gr.Slider,
#         gr.Checkbox,
#         gr.Radio,
#         gr.Textbox,
#         gr.Image,
#         gr.Gallery,
#         gr.Audio,
#         gr.Video,
#         gr.File,
#     ]
#     states = {}

#     def save_state(v, i):
#         states[i] = v

#     def load_state(i):
#         return states.get(i, None)

#     def actual_func(*args, **kwargs):
#         inst = init_func(*args, **kwargs)
#         iface = inst.iface
#         ignore_elements = inst.ignore_elements
#         ignore_indexes = [element._id for element in ignore_elements]
#         with iface:
#             indexes = []

#             for index, block in iface.blocks.items():
#                 if (
#                     any([isinstance(block, dtype) for dtype in dtypes])
#                     and index not in ignore_indexes
#                 ):
#                     indexes.append(index)

#             for index in indexes:
#                 block = iface.blocks[index]
#                 block.change(
#                     fn=lambda x: save_state(x, f"blockid-{index}"), inputs=[block]
#                 )
#                 iface.load(fn=lambda: load_state(f"blockid-{index}"), outputs=[block])

#     return actual_func



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from PIL import Image
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.beit import BeitProcessor as _BeitProcessor
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli import WriterOutputs
from unitorch.cli.models import (
    TensorsInputs,
    GenerationOutputs,
    GenerationTargets,
)
from unitorch.cli.models.beit import pretrained_beit_infos


class BeitProcessor(_BeitProcessor):
    """Class for processing images using the Beit model."""

    def __init__(
        self,
        vision_config_path: str,
    ):
        """
        Initialize BeitProcessor.

        Args:
            vision_config_path (str): The path to the vision configuration file.
        """
        super().__init__(
            vision_config_path=vision_config_path,
        )

    @classmethod
    @add_default_section_for_init("core/process/beit")
    def from_core_configure(cls, config, **kwargs):
        """
        Create an instance of BeitProcessor from a core configuration.

        Args:
            config: The core configuration.
            **kwargs: Additional keyword arguments.

        Returns:
            BeitProcessor: An instance of BeitProcessor.
        """
        config.set_default_section("core/process/beit")
        pretrained_name = config.getoption("pretrained_name", "default-beit")
        vision_config_path = config.getoption("vision_config_path", None)
        vision_config_path = pop_value(
            vision_config_path,
            nested_dict_value(pretrained_beit_infos, pretrained_name, "vision_config"),
        )
        vision_config_path = cached_path(vision_config_path)

        return {
            "vision_config_path": vision_config_path,
        }

    @register_process("core/process/beit/classification")
    def _classification(
        self,
        image: Union[Image.Image, str],
    ):
        """
        Perform image classification using BeitProcessor.

        Args:
            image (Union[Image.Image, str]): The input image.

        Returns:
            TensorsInputs: The input tensors for the model.
        """
        if isinstance(image, str):
            image = Image.open(image)
        outputs = super().classification(image=image)
        return TensorsInputs(pixel_values=outputs.pixel_values)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch.optim as optim
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.schedulers.warmup import (
    CosineWarmupScheduler,
    LinearWarmupScheduler,
)
from unitorch.models import CheckpointMixin
from unitorch.cli import add_default_section_for_init, register_scheduler


@register_scheduler("core/scheduler/cosine_warmup")
class CosineWarmupScheduler(CosineWarmupScheduler, CheckpointMixin):
    def __init__(
        self,
        optimizer: optim.Optimizer,
        num_warmup_steps: int,
        num_training_steps: int,
        num_cycles: Optional[float] = 0.5,
        last_epoch: Optional[int] = -1,
    ):
        super().__init__(
            optimizer=optimizer,
            num_warmup_steps=num_warmup_steps,
            num_training_steps=num_training_steps,
            num_cycles=num_cycles,
            last_epoch=last_epoch,
        )

    @classmethod
    @add_default_section_for_init("core/scheduler/cosine_warmup")
    def from_core_configure(cls, config, **kwargs):
        num_warmup_steps = config.getdefault(
            "core/scheduler/cosine_warmup", "num_warmup_steps", -1
        )
        num_warmup_rate = config.getdefault(
            "core/scheduler/cosine_warmup", "num_warmup_rate", 0.001
        )
        num_cycles = config.getdefault(
            "core/scheduler/cosine_warmup", "num_cycles", 0.5
        )
        num_training_steps = kwargs.get("num_training_steps", 1000000)
        if num_warmup_steps < 0:
            num_warmup_steps = int(num_training_steps * num_warmup_rate)
        return dict(
            {
                "num_warmup_steps": num_warmup_steps,
                "num_training_steps": num_training_steps,
                "num_cycles": num_cycles,
            }
        )


@register_scheduler("core/scheduler/linear_warmup")
class LinearWarmupScheduler(LinearWarmupScheduler, CheckpointMixin):
    def __init__(
        self,
        optimizer: optim.Optimizer,
        num_warmup_steps: int,
        num_training_steps: int,
        last_epoch: Optional[int] = -1,
    ):
        super().__init__(
            optimizer=optimizer,
            num_warmup_steps=num_warmup_steps,
            num_training_steps=num_training_steps,
            last_epoch=last_epoch,
        )

    @classmethod
    @add_default_section_for_init("core/scheduler/linear_warmup")
    def from_core_configure(cls, config, **kwargs):
        num_warmup_steps = config.getdefault(
            "core/scheduler/linear_warmup", "num_warmup_steps", -1
        )
        num_warmup_rate = config.getdefault(
            "core/scheduler/linear_warmup", "num_warmup_rate", 0.001
        )
        num_training_steps = kwargs.get("num_training_steps", 1000000)
        if num_warmup_steps < 0:
            num_warmup_steps = int(num_training_steps * num_warmup_rate)
        return dict(
            {
                "num_warmup_steps": num_warmup_steps,
                "num_training_steps": num_training_steps,
            }
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import io
import torch
import gc
import gradio as gr
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.cli import CoreConfigureParser, GenericWebUI
from unitorch.cli import register_webui
from unitorch.cli.webuis import SimpleWebUI
from unitorch.cli.webuis.stable_xl.text2image import StableXLText2ImageWebUI
from unitorch.cli.webuis.stable_xl.image2image import StableXLImage2ImageWebUI
from unitorch.cli.webuis.stable_xl.inpainting import StableXLImageInpaintingWebUI


@register_webui("core/webui/stable_xl")
class StableXLWebUI(SimpleWebUI):
    def __init__(self, config: CoreConfigureParser):
        webuis = [
            StableXLText2ImageWebUI(config),
            StableXLImage2ImageWebUI(config),
            StableXLImageInpaintingWebUI(config),
        ]
        iface = gr.TabbedInterface(
            [webui.iface for webui in webuis],
            tab_names=[webui.iname for webui in webuis],
        )
        super().__init__(config, iname="StableXL", iface=iface)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import io
import torch
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from fastapi import APIRouter
from fastapi.responses import StreamingResponse
from PIL import Image
from unitorch.cli import CoreConfigureParser, GenericFastAPI
from unitorch.cli import register_fastapi
from unitorch.cli.pipelines.stable import StableForText2ImageGenerationPipeline


@register_fastapi("core/fastapi/stable/text2image")
class StableText2ImageFastAPI(GenericFastAPI):
    def __init__(self, config: CoreConfigureParser):
        self.config = config
        config.set_default_section(f"core/fastapi/stable/text2image")
        router = config.getoption("router", "core/fastapi/stable/text2image")
        self._pipe = None if not hasattr(self, "_pipe") else self._pipe
        self._router = APIRouter(prefix=router)
        self._router.add_api_route("/", self.serve, methods=["GET"])

    @property
    def router(self):
        return self._router

    def start(self, **kwargs):
        self._pipe = StableForText2ImageGenerationPipeline.from_core_configure(
            self.config
        )

    def stop(self, **kwargs):
        del self._pipe
        self._pipe = None if not hasattr(self, "_pipe") else self._pipe

    def serve(
        self,
        text: str,
        height: Optional[int] = 512,
        width: Optional[int] = 512,
        guidance_scale: Optional[float] = 7.5,
        num_timesteps: Optional[int] = 50,
        seed: Optional[int] = 1123,
    ):
        assert self._pipe is not None
        image = self._pipe(
            text,
            height=height,
            width=width,
            guidance_scale=guidance_scale,
            num_timesteps=num_timesteps,
            seed=seed,
        )
        buffer = io.BytesIO()
        image.save(buffer, format="PNG")

        return StreamingResponse(
            io.BytesIO(buffer.getvalue()),
            media_type="image/png",
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

# pretrained infos
pretrained_bloom_infos = {
    # bloom
    "default-bloom": {
        "config": "https://huggingface.co/bigscience/bloom-560m/resolve/main/config.json",
        "tokenizer": "https://huggingface.co/bigscience/bloom-560m/resolve/main/tokenizer.json",
    },
    "bloom-560m": {
        "config": "https://huggingface.co/bigscience/bloom-560m/resolve/main/config.json",
        "tokenizer": "https://huggingface.co/bigscience/bloom-560m/resolve/main/tokenizer.json",
        "weight": "https://huggingface.co/bigscience/bloom-560m/resolve/main/pytorch_model.bin",
    },
    "bloom-3b": {
        "config": "https://huggingface.co/bigscience/bloom-3b/resolve/main/config.json",
        "tokenizer": "https://huggingface.co/bigscience/bloom-3b/resolve/main/tokenizer.json",
        "weight": "https://huggingface.co/bigscience/bloom-3b/resolve/main/pytorch_model.bin",
    },
    "bloom-7b1": {
        "config": "https://huggingface.co/bigscience/bloom-7b1/resolve/main/config.json",
        "tokenizer": "https://huggingface.co/bigscience/bloom-7b1/resolve/main/tokenizer.json",
        "weight": [
            f"https://huggingface.co/bigscience/bloom-7b1/resolve/main/pytorch_model-{str(i).rjust(5, '0')}-of-00002.bin"
            for i in range(1, 3)
        ],
    },
    "bloomz-560m": {
        "config": "https://huggingface.co/bigscience/bloomz-560m/resolve/main/config.json",
        "tokenizer": "https://huggingface.co/bigscience/bloomz-560m/resolve/main/tokenizer.json",
        "weight": "https://huggingface.co/bigscience/bloomz-560m/resolve/main/pytorch_model.bin",
    },
    "bloomz-3b": {
        "config": "https://huggingface.co/bigscience/bloomz-3b/resolve/main/config.json",
        "tokenizer": "https://huggingface.co/bigscience/bloomz-3b/resolve/main/tokenizer.json",
        "weight": "https://huggingface.co/bigscience/bloomz-3b/resolve/main/pytorch_model.bin",
    },
    "bloomz-7b1": {
        "config": "https://huggingface.co/bigscience/bloomz-7b1/resolve/main/config.json",
        "tokenizer": "https://huggingface.co/bigscience/bloomz-7b1/resolve/main/tokenizer.json",
        "weight": "https://huggingface.co/bigscience/bloomz-7b1/resolve/main/pytorch_model.bin",
    },
}

pretrained_bloom_extensions_infos = {
}

import unitorch.cli.models.bloom.modeling
import unitorch.cli.models.bloom.processing
from unitorch.cli.models.bloom.modeling import (
    BloomForClassification,
    BloomForGeneration,
)
from unitorch.cli.models.bloom.processing import BloomProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

# pretrained infos
pretrained_mask2former_infos = {
    "default-mask2former": {
        "config": "https://huggingface.co/facebook/mask2former-swin-large-ade-semantic/resolve/main/config.json",
        "vision_config": "https://huggingface.co/facebook/mask2former-swin-large-ade-semantic/resolve/main/preprocessor_config.json",
    },
    "mask2former-swin-tiny-ade-semantic": {
        "config": "https://huggingface.co/facebook/mask2former-swin-tiny-ade-semantic/resolve/main/config.json",
        "vision_config": "https://huggingface.co/facebook/mask2former-swin-tiny-ade-semantic/resolve/main/preprocessor_config.json",
        "weight": "https://huggingface.co/facebook/mask2former-swin-tiny-ade-semantic/resolve/main/pytorch_model.bin",
    },
    "mask2former-swin-base-ade-semantic": {
        "config": "https://huggingface.co/facebook/mask2former-swin-base-ade-semantic/resolve/main/config.json",
        "vision_config": "https://huggingface.co/facebook/mask2former-swin-base-ade-semantic/resolve/main/preprocessor_config.json",
        "weight": "https://huggingface.co/facebook/mask2former-swin-base-ade-semantic/resolve/main/pytorch_model.bin",
    },
    "mask2former-swin-large-ade-semantic": {
        "config": "https://huggingface.co/facebook/mask2former-swin-large-ade-semantic/resolve/main/config.json",
        "vision_config": "https://huggingface.co/facebook/mask2former-swin-large-ade-semantic/resolve/main/preprocessor_config.json",
        "weight": "https://huggingface.co/facebook/mask2former-swin-large-ade-semantic/resolve/main/pytorch_model.bin",
    },
}

import unitorch.cli.models.mask2former.modeling
import unitorch.cli.models.mask2former.processing
from unitorch.cli.models.mask2former.modeling import Mask2FormerForSegmentation
from unitorch.cli.models.mask2former.processing import Mask2FormerProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import numpy as np
from PIL import Image, ImageFilter
from typing import Any, Dict, List, Optional, Tuple, Union
from unitorch.utils import is_opencv_available
from unitorch.cli import CoreConfigureParser
from unitorch.cli.pipelines.dpt import DPTForDepthEstimationPipeline


def canny(image: Image.Image):
    if is_opencv_available():
        import cv2

        image = np.array(image, np.uint8)
        image = cv2.cvtColor(image, cv2.COLOR_RGB2GRAY)
        image = cv2.Canny(image, 100, 200)
        image = Image.fromarray(image)
    else:
        image = image.convert("L")
        image = image.filter(ImageFilter.FIND_EDGES)
    return image


dpt_pipe = None


def depth(image: Image.Image):
    global dpt_pipe
    if dpt_pipe is None:
        dpt_pipe = DPTForDepthEstimationPipeline.from_core_configure(
            CoreConfigureParser(), pretrained_name="dpt-large"
        )
        dpt_pipe.to("cpu")
    return dpt_pipe(image)


controlnet_processes = {
    "canny": canny,
    "depth": depth,
}

adapter_processes = {
    "canny": canny,
    "depth": depth,
}

from unitorch.cli.pipelines.stable_xl.text2image import (
    StableXLForText2ImageGenerationPipeline,
)
from unitorch.cli.pipelines.stable_xl.image2image import (
    StableXLForImage2ImageGenerationPipeline,
)
from unitorch.cli.pipelines.stable_xl.inpainting import (
    StableXLForImageInpaintingPipeline,
)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import re
import torch
import numpy as np
from PIL import Image, ImageDraw
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.bria import (
    BRIAProcessor,
    BRIAForSegmentation as _BRIAForSegmentation,
)
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
)


class BRIAForSegmentationPipeline(_BRIAForSegmentation):
    def __init__(
        self,
        in_channels: Optional[int] = 3,
        out_channels: Optional[int] = 1,
        image_size: Optional[int] = 1024,
        weight_path: Optional[Union[str, List[str]]] = None,
        state_dict: Optional[Dict[str, Any]] = None,
        device: Optional[Union[str, int]] = "cpu",
    ):
        super().__init__(
            in_ch=in_channels,
            out_ch=out_channels,
        )
        self.processor = BRIAProcessor(
            image_size=image_size,
        )
        self._device = "cpu" if device == "cpu" else int(device)

        self.from_pretrained(weight_path, state_dict=state_dict)
        self.to(device=self._device)
        self.eval()

    @classmethod
    @add_default_section_for_init("core/pipeline/bria")
    def from_core_configure(
        cls,
        config,
        pretrained_weight_path: Optional[str] = None,
        device: Optional[str] = "cpu",
        **kwargs,
    ):
        config.set_default_section("core/pipeline/bria")

        in_channels = config.getoption("in_channels", 3)
        out_channels = config.getoption("out_channels", 1)
        image_size = config.getoption("image_size", 1024)
        device = config.getoption("device", device)
        weight_path = config.getoption("pretrained_weight_path", pretrained_weight_path)

        inst = cls(
            in_channels=in_channels,
            out_channels=out_channels,
            image_size=image_size,
            weight_path=weight_path,
            device=device,
        )

        return inst

    @torch.no_grad()
    @add_default_section_for_function("core/pipeline/bria")
    def __call__(
        self,
        image: Union[Image.Image, str],
        threshold: Optional[float] = 0.5,
    ):
        inputs = self.processor.segmentation_inputs(image)
        pixel_values = inputs.image.unsqueeze(0).to(self._device)
        outputs = self.forward(
            pixel_values,
        )
        masks = [
            (mask.squeeze(0).cpu().numpy() > threshold).astype(np.uint8)
            for mask in outputs
        ][0]
        result_image = Image.fromarray(masks * 255)
        result_image = result_image.resize(image.size)

        return result_image



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import abc
import warnings
import logging
import torch
import torch.nn as nn
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.utils import replace, is_onnxruntime_available
from unitorch.models import GenericModel


class GenericOnnxModel(GenericModel):
    providers = (
        ["CUDAExecutionProvider", "CPUExecutionProvider"]
        if torch.cuda.is_available()
        else ["CPUExecutionProvider"]
    )

    def __init__(self, *args, **kwargs):
        pass

    @abc.abstractmethod
    def preprocess(self, *args, **kwargs):
        pass

    @abc.abstractmethod
    def forward(self, *args, **kwargs):
        pass


if is_onnxruntime_available():
    pass



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import random
from functools import partial
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union

import numpy as np
import torch

from transformers import LlamaTokenizer
from unitorch.utils import pop_value, truncate_sequence_pair
from unitorch.models import (
    HfTextClassificationProcessor,
    HfTextGenerationProcessor,
    GenericOutputs,
)


class MistralProcessor(HfTextClassificationProcessor, HfTextGenerationProcessor):
    def __init__(
        self,
        vocab_file: str,
        max_seq_length: Optional[int] = 128,
        max_gen_seq_length: Optional[int] = 48,
    ):
        """
        Initialize the MistralProcessor.

        Args:
            vocab_file (str): Path to the vocabulary file.
            max_seq_length (int, optional): Maximum sequence length for text classification. Defaults to 128.
            max_gen_seq_length (int, optional): Maximum sequence length for text generation. Defaults to 48.
        """
        tokenizer = LlamaTokenizer(vocab_file=vocab_file)
        tokenizer.cls_token = tokenizer.bos_token
        tokenizer.sep_token = tokenizer.eos_token
        tokenizer.pad_token = tokenizer.unk_token
        tokenizer.cls_token_id = tokenizer.bos_token_id
        tokenizer.sep_token_id = tokenizer.eos_token_id
        tokenizer.pad_token_id = tokenizer.unk_token_id
        HfTextClassificationProcessor.__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
        )
        HfTextGenerationProcessor.__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )

    def classification(
        self,
        text: str,
        text_pair: Optional[str] = None,
        max_seq_length: Optional[int] = None,
    ):
        """
        Process text for classification.

        Args:
            text (str): Input text.
            text_pair (str, optional): Input text pair. Defaults to None.
            max_seq_length (int, optional): Maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids and attention_mask tensors.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )

        tokens = self.tokenizer.tokenize(str(text))
        if text_pair is None:
            tokens = tokens[:max_seq_length]
            input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
        else:
            tokens_pair = self.tokenizer.tokenize(str(text_pair))
            truncate_sequence_pair(tokens, tokens_pair, max_seq_length)
            tokens = tokens + tokens_pair
            input_ids = self.tokenizer.convert_tokens_to_ids(tokens)

        padding = [0] * (max_seq_length - len(input_ids))
        attention_mask = [0] * len(padding) + [1] * len(input_ids)
        input_ids = len(padding) * [self.pad_token_id] + input_ids

        assert len(input_ids) == max_seq_length
        assert len(attention_mask) == max_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
        )

    def generation_inputs(
        self,
        text: str,
        max_seq_length: Optional[int] = None,
    ):
        """
        Process text for generation inputs.

        Args:
            text (str): Input text.
            max_seq_length (int, optional): Maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids tensor.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )
        tokens = [self.bos_token] + self.tokenizer.tokenize(str(text))[
            1 - max_seq_length :
        ]
        padding = [self.pad_token] * (max_seq_length - len(tokens))
        tokens = padding + tokens
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)

        assert len(input_ids) == max_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
        )

    def generation_labels(
        self,
        text: str,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Process text for generation labels.

        Args:
            text (str): Input text.
            max_gen_seq_length (int, optional): Maximum generation sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids and attention_mask tensors.
        """
        max_gen_seq_length = pop_value(
            max_gen_seq_length,
            self.max_gen_seq_length,
        )
        tokens = self.tokenizer.tokenize(str(text))[: max_gen_seq_length - 1] + [
            self.eos_token
        ]
        padding = [self.pad_token] * (max_gen_seq_length - len(tokens))
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
        attention_mask = [1] * len(input_ids)

        padding = [0] * (max_gen_seq_length - len(input_ids))
        input_ids += [self.pad_token_id] * len(padding)
        attention_mask += padding

        assert len(input_ids) == max_gen_seq_length
        assert len(attention_mask) == max_gen_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
        )

    def generation(
        self,
        text: str,
        text_pair: str,
        max_seq_length: Optional[int] = None,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Process text for generation.

        Args:
            text (str): Input text.
            text_pair (str): Input text pair.
            max_seq_length (int, optional): Maximum sequence length. Defaults to None.
            max_gen_seq_length (int, optional): Maximum generation sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids, attention_mask, input_ids_label, and attention_mask_label tensors.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )
        max_gen_seq_length = pop_value(
            max_gen_seq_length,
            self.max_gen_seq_length,
        )

        tokens = [self.bos_token] + self.tokenizer.tokenize(str(text))[
            1 - max_seq_length :
        ]
        tokens_pair = self.tokenizer.tokenize(str(text_pair))[
            : max_gen_seq_length - 1
        ] + [self.eos_token]
        padding_a = [self.pad_token] * (max_seq_length - len(tokens))
        padding_b = [self.pad_token] * (max_gen_seq_length - len(tokens_pair))
        attention_mask = (
            [0] * len(padding_a)
            + [1] * (len(tokens) + len(tokens_pair))
            + [0] * len(padding_b)
        )
        _tokens = padding_a + tokens + tokens_pair + padding_b
        input_ids = self.tokenizer.convert_tokens_to_ids(_tokens)

        tokens_label = tokens_pair + [self.pad_token] * (
            max_gen_seq_length - len(tokens_pair) + 1
        )
        input_ids_label = self.tokenizer.convert_tokens_to_ids(tokens_label)
        input_ids_label = [0] * (max_seq_length - 1) + input_ids_label
        attention_mask_label = [1] * len(tokens_pair) + [0] * (
            max_gen_seq_length - len(tokens_pair) + 1
        )
        attention_mask_label = [0] * (max_seq_length - 1) + attention_mask_label

        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
            input_ids_label=torch.tensor(input_ids_label, dtype=torch.long),
            attention_mask_label=torch.tensor(attention_mask_label, dtype=torch.long),
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import re
import torch
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.models.clip import (
    ClipForClassification as _ClipForClassification,
    ClipForTextClassification as _ClipForTextClassification,
    ClipForImageClassification as _ClipForImageClassification,
)
from unitorch.models.clip import ClipProcessor
from unitorch.utils import pop_value, nested_dict_value
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
)
from unitorch.cli.models.clip import pretrained_clip_infos


class ClipForClassificationPipeline(_ClipForClassification):
    def __init__(
        self,
        config_path: str,
        vocab_path: str,
        merge_path: str,
        vision_config_path: str,
        projection_dim: Optional[int] = 512,
        num_classes: Optional[int] = 1,
        max_seq_length: Optional[int] = 512,
        id2label: Optional[Dict[int, str]] = None,
        weight_path: Optional[Union[str, List[str]]] = None,
        state_dict: Optional[Dict[str, Any]] = None,
        device: Optional[Union[str, int]] = "cpu",
    ):
        super().__init__(
            config_path=config_path,
            projection_dim=projection_dim,
            num_classes=num_classes,
        )
        self.processor = ClipProcessor(
            vocab_path=vocab_path,
            merge_path=merge_path,
            vision_config_path=vision_config_path,
            max_seq_length=max_seq_length,
        )
        self.id2label = id2label
        self._device = "cpu" if device == "cpu" else int(device)

        self.from_pretrained(weight_path, state_dict=state_dict)
        self.to(device=self._device)
        self.eval()

    @classmethod
    @add_default_section_for_init("core/pipeline/clip")
    def from_core_configure(
        cls,
        config,
        pretrained_name: Optional[str] = "default-clip",
        config_path: Optional[str] = None,
        vocab_path: Optional[str] = None,
        merge_path: Optional[str] = None,
        vision_config_path: Optional[str] = None,
        id2label: Optional[Dict[int, str]] = None,
        pretrained_weight_path: Optional[str] = None,
        device: Optional[str] = "cpu",
        **kwargs,
    ):
        config.set_default_section("core/pipeline/clip")
        pretrained_name = config.getoption("pretrained_name", pretrained_name)

        config_path = config.getoption("config_path", config_path)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_clip_infos, pretrained_name, "config"),
        )
        config_path = cached_path(config_path)

        vocab_path = config.getoption("vocab_path", vocab_path)
        vocab_path = pop_value(
            vocab_path,
            nested_dict_value(pretrained_clip_infos, pretrained_name, "vocab"),
        )
        vocab_path = cached_path(vocab_path)

        merge_path = config.getoption("merge_path", merge_path)
        merge_path = pop_value(
            merge_path,
            nested_dict_value(pretrained_clip_infos, pretrained_name, "merge"),
        )
        merge_path = cached_path(merge_path)

        vision_config_path = config.getoption("vision_config_path", vision_config_path)
        vision_config_path = pop_value(
            vision_config_path,
            nested_dict_value(pretrained_clip_infos, pretrained_name, "vision_config"),
        )

        vision_config_path = cached_path(vision_config_path)

        projection_dim = config.getoption("projection_dim", 512)
        num_classes = config.getoption("num_classes", 1)
        max_seq_length = config.getoption("max_seq_length", 512)
        id2label = config.getoption("id2label", id2label)

        device = config.getoption("device", device)
        pretrained_weight_path = config.getoption(
            "pretrained_weight_path", pretrained_weight_path
        )
        weight_path = pop_value(
            pretrained_weight_path,
            nested_dict_value(pretrained_clip_infos, pretrained_name, "weight"),
            check_none=False,
        )

        inst = cls(
            config_path,
            vocab_path=vocab_path,
            merge_path=merge_path,
            vision_config_path=vision_config_path,
            projection_dim=projection_dim,
            num_classes=num_classes,
            max_seq_length=max_seq_length,
            id2label=id2label,
            weight_path=weight_path,
            device=device,
        )

        return inst

    @torch.no_grad()
    @add_default_section_for_function("core/pipeline/clip")
    def __call__(
        self,
        text: str,
        image: Image.Image,
        max_seq_length: Optional[int] = 512,
    ):
        inputs = self.processor.classification(
            text=text,
            image=image,
            max_seq_length=max_seq_length,
        )
        inputs = {k: v.unsqueeze(0) if v is not None else v for k, v in inputs.items()}
        inputs = {
            k: v.to(device=self._device) if v is not None else v
            for k, v in inputs.items()
        }
        outputs = super().forward(
            input_ids=inputs["input_ids"],
            attention_mask=inputs["attention_mask"],
            pixel_values=inputs["pixel_values"],
        )
        scores = outputs.softmax(dim=-1).squeeze(0)
        if self.id2label is not None:
            return self.id2label[scores.argmax(-1).item()], scores.max(-1)[0].item()
        return scores.argmax(-1).item(), scores.max(-1)[0].item()


class ClipForTextClassificationPipeline(_ClipForTextClassification):
    def __init__(
        self,
        config_path: str,
        vocab_path: str,
        merge_path: str,
        projection_dim: Optional[int] = 512,
        num_classes: Optional[int] = 1,
        max_seq_length: Optional[int] = 512,
        id2label: Optional[Dict[int, str]] = None,
        weight_path: Optional[Union[str, List[str]]] = None,
        state_dict: Optional[Dict[str, Any]] = None,
        device: Optional[Union[str, int]] = "cpu",
    ):
        super().__init__(
            config_path=config_path,
            projection_dim=projection_dim,
            num_classes=num_classes,
        )
        self.processor = ClipProcessor(
            vocab_path=vocab_path,
            merge_path=merge_path,
            max_seq_length=max_seq_length,
        )
        self.id2label = id2label
        self._device = "cpu" if device == "cpu" else int(device)

        self.from_pretrained(weight_path, state_dict=state_dict)
        self.to(device=self._device)
        self.eval()

    @classmethod
    @add_default_section_for_init("core/pipeline/clip/text")
    def from_core_configure(
        cls,
        config,
        pretrained_name: Optional[str] = "default-clip",
        config_path: Optional[str] = None,
        vocab_path: Optional[str] = None,
        merge_path: Optional[str] = None,
        id2label: Optional[Dict[int, str]] = None,
        pretrained_weight_path: Optional[str] = None,
        device: Optional[str] = "cpu",
        **kwargs,
    ):
        config.set_default_section("core/pipeline/clip/text")
        pretrained_name = config.getoption("pretrained_name", pretrained_name)

        config_path = config.getoption("config_path", config_path)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_clip_infos, pretrained_name, "config"),
        )
        config_path = cached_path(config_path)

        vocab_path = config.getoption("vocab_path", vocab_path)
        vocab_path = pop_value(
            vocab_path,
            nested_dict_value(pretrained_clip_infos, pretrained_name, "vocab"),
        )
        vocab_path = cached_path(vocab_path)

        merge_path = config.getoption("merge_path", merge_path)
        merge_path = pop_value(
            merge_path,
            nested_dict_value(pretrained_clip_infos, pretrained_name, "merge"),
        )
        merge_path = cached_path(merge_path)

        projection_dim = config.getoption("projection_dim", 512)
        num_classes = config.getoption("num_classes", 1)
        max_seq_length = config.getoption("max_seq_length", 512)
        id2label = config.getoption("id2label", id2label)

        device = config.getoption("device", device)
        pretrained_weight_path = config.getoption(
            "pretrained_weight_path", pretrained_weight_path
        )
        weight_path = pop_value(
            pretrained_weight_path,
            nested_dict_value(pretrained_clip_infos, pretrained_name, "weight"),
            check_none=False,
        )

        inst = cls(
            config_path,
            vocab_path=vocab_path,
            merge_path=merge_path,
            projection_dim=projection_dim,
            num_classes=num_classes,
            max_seq_length=max_seq_length,
            id2label=id2label,
            weight_path=weight_path,
            device=device,
        )

        return inst

    @torch.no_grad()
    @add_default_section_for_function("core/pipeline/clip/text")
    def __call__(
        self,
        text: str,
        max_seq_length: Optional[int] = 512,
    ):
        inputs = self.processor.text_classification(
            text=text,
            max_seq_length=max_seq_length,
        )
        inputs = {k: v.unsqueeze(0) if v is not None else v for k, v in inputs.items()}
        inputs = {
            k: v.to(device=self._device) if v is not None else v
            for k, v in inputs.items()
        }
        outputs = super().forward(
            input_ids=inputs["input_ids"],
            attention_mask=inputs["attention_mask"],
        )
        scores = outputs.softmax(dim=-1).squeeze(0)
        if self.id2label is not None:
            return self.id2label[scores.argmax(-1).item()], scores.max(-1)[0].item()
        return scores.argmax(-1).item(), scores.max(-1)[0].item()


class ClipForImageClassificationPipeline(_ClipForImageClassification):
    def __init__(
        self,
        config_path: str,
        vision_config_path: str,
        projection_dim: Optional[int] = 512,
        num_classes: Optional[int] = 1,
        id2label: Optional[Dict[int, str]] = None,
        weight_path: Optional[Union[str, List[str]]] = None,
        state_dict: Optional[Dict[str, Any]] = None,
        device: Optional[Union[str, int]] = "cpu",
    ):
        super().__init__(
            config_path=config_path,
            projection_dim=projection_dim,
            num_classes=num_classes,
        )
        self.processor = ClipProcessor(
            vision_config_path=vision_config_path,
        )
        self.id2label = id2label
        self._device = "cpu" if device == "cpu" else int(device)

        self.from_pretrained(weight_path, state_dict=state_dict)
        self.to(device=self._device)
        self.eval()

    @classmethod
    @add_default_section_for_init("core/pipeline/clip/image")
    def from_core_configure(
        cls,
        config,
        pretrained_name: Optional[str] = "default-clip",
        config_path: Optional[str] = None,
        vision_config_path: Optional[str] = None,
        id2label: Optional[Dict[int, str]] = None,
        pretrained_weight_path: Optional[str] = None,
        device: Optional[str] = "cpu",
        **kwargs,
    ):
        config.set_default_section("core/pipeline/clip/image")
        pretrained_name = config.getoption("pretrained_name", pretrained_name)

        config_path = config.getoption("config_path", config_path)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_clip_infos, pretrained_name, "config"),
        )
        config_path = cached_path(config_path)

        vision_config_path = config.getoption("vision_config_path", vision_config_path)
        vision_config_path = pop_value(
            vision_config_path,
            nested_dict_value(pretrained_clip_infos, pretrained_name, "vision_config"),
        )

        vision_config_path = cached_path(vision_config_path)

        projection_dim = config.getoption("projection_dim", 512)
        num_classes = config.getoption("num_classes", 1)
        max_seq_length = config.getoption("max_seq_length", 512)
        id2label = config.getoption("id2label", id2label)

        device = config.getoption("device", device)
        pretrained_weight_path = config.getoption(
            "pretrained_weight_path", pretrained_weight_path
        )
        weight_path = pop_value(
            pretrained_weight_path,
            nested_dict_value(pretrained_clip_infos, pretrained_name, "weight"),
            check_none=False,
        )

        inst = cls(
            config_path,
            vision_config_path=vision_config_path,
            projection_dim=projection_dim,
            num_classes=num_classes,
            max_seq_length=max_seq_length,
            id2label=id2label,
            weight_path=weight_path,
            device=device,
        )

        return inst

    @torch.no_grad()
    @add_default_section_for_function("core/pipeline/clip/image")
    def __call__(
        self,
        image: Image.Image,
    ):
        inputs = self.processor.image_classification(
            image=image,
        )
        inputs = {k: v.unsqueeze(0) if v is not None else v for k, v in inputs.items()}
        inputs = {
            k: v.to(device=self._device) if v is not None else v
            for k, v in inputs.items()
        }
        outputs = super().forward(
            pixel_values=inputs["pixel_values"],
        )
        scores = outputs.softmax(dim=-1).squeeze(0)
        if self.id2label is not None:
            return self.id2label[scores.argmax(-1).item()], scores.max(-1)[0].item()
        return scores.argmax(-1).item(), scores.max(-1)[0].item()



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import torch
import json
import numpy as np
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from transformers import CLIPTokenizer, T5Tokenizer
from torchvision.transforms import (
    Resize,
    CenterCrop,
    RandomCrop,
    Lambda,
    ToTensor,
    Normalize,
    Compose,
    RandomHorizontalFlip,
)
from torchvision.transforms.functional import crop
from diffusers.image_processor import VaeImageProcessor
from unitorch.models import HfTextClassificationProcessor, GenericOutputs


class Stable3Processor:
    def __init__(
        self,
        vocab_path: str,
        merge_path: str,
        vocab2_path: str,
        merge2_path: str,
        vocab3_path: str,
        vae_config_path: Optional[str] = None,
        max_seq_length: Optional[int] = 77,
        max_seq_length2: Optional[int] = 256,
        position_start_id: Optional[int] = 0,
        pad_token: Optional[str] = "<|endoftext|>",
        pad_token2: Optional[str] = "!",
        image_size: Optional[Tuple[int, int]] = None,
        center_crop: Optional[bool] = False,
        random_flip: Optional[bool] = False,
    ):
        tokenizer1 = CLIPTokenizer(
            vocab_file=vocab_path,
            merges_file=merge_path,
        )

        tokenizer1.cls_token = tokenizer1.bos_token
        tokenizer1.sep_token = tokenizer1.eos_token
        tokenizer1.pad_token = pad_token

        self.text_processor1 = HfTextClassificationProcessor(
            tokenizer=tokenizer1,
            max_seq_length=max_seq_length,
            position_start_id=position_start_id,
        )

        tokenizer2 = CLIPTokenizer(
            vocab_file=vocab2_path,
            merges_file=merge2_path,
        )

        tokenizer2.cls_token = tokenizer2.bos_token
        tokenizer2.sep_token = tokenizer2.eos_token
        tokenizer2.pad_token = pad_token2

        self.text_processor2 = HfTextClassificationProcessor(
            tokenizer=tokenizer2,
            max_seq_length=max_seq_length,
            position_start_id=position_start_id,
        )

        tokenizer3 = T5Tokenizer(
            vocab3_path,
        )
        tokenizer3.bos_token_id = 0
        tokenizer3.cls_token = tokenizer3.convert_ids_to_tokens(0)
        tokenizer3.sep_token = tokenizer3.eos_token
        tokenizer3.sep_token_id = tokenizer3.eos_token_id

        self.text_processor3 = HfTextClassificationProcessor(
            tokenizer=tokenizer3,
            max_seq_length=max_seq_length2,
            position_start_id=position_start_id,
        )

        self.image_size = image_size

        if self.image_size is not None:
            self.vision_processor = Compose(
                [
                    Resize(self.image_size),
                    CenterCrop(self.image_size)
                    if center_crop
                    else RandomCrop(self.image_size),
                    RandomHorizontalFlip() if random_flip else Lambda(lambda x: x),
                    ToTensor(),
                    Normalize([0.5], [0.5]),
                ]
            )

            self.condition_vision_processor = Compose(
                [
                    Resize(self.image_size),
                    CenterCrop(self.image_size),
                    ToTensor(),
                ]
            )
        else:
            self.vision_processor = Compose(
                [
                    RandomHorizontalFlip() if random_flip else Lambda(lambda x: x),
                    ToTensor(),
                    Normalize([0.5], [0.5]),
                ]
            )

            self.condition_vision_processor = Compose(
                [
                    ToTensor(),
                ]
            )

        if vae_config_path is not None:
            vae_config_dict = json.load(open(vae_config_path))
            vae_scale_factor = 2 ** (
                len(vae_config_dict.get("block_out_channels", [])) - 1
            )
            self.vae_image_processor = VaeImageProcessor(
                vae_scale_factor=vae_scale_factor
            )
            self.vae_mask_image_processor = VaeImageProcessor(
                vae_scale_factor=vae_scale_factor
            )
            self.vae_condition_image_processor = VaeImageProcessor(
                vae_scale_factor=vae_scale_factor,
                do_convert_rgb=True,
                do_normalize=False,
            )
        else:
            self.vae_image_processor = None
            self.vae_mask_image_processor = None
            self.vae_condition_image_processor = None

    def text2image(
        self,
        prompt: str,
        image: Union[Image.Image, str],
        prompt2: Optional[str] = None,
        prompt3: Optional[str] = None,
        max_seq_length: Optional[int] = None,
        max_seq_length2: Optional[int] = None,
    ):
        if isinstance(image, str):
            image = Image.open(image)
        image = image.convert("RGB")
        pixel_values = self.vision_processor(image)

        prompt2 = prompt2 or prompt
        prompt3 = prompt3 or prompt
        prompt_outputs = self.text_processor1.classification(
            prompt, max_seq_length=max_seq_length
        )
        prompt2_outputs = self.text_processor2.classification(
            prompt2, max_seq_length=max_seq_length
        )
        prompt3_outputs = self.text_processor3.classification(
            prompt3, max_seq_length=max_seq_length2
        )

        return GenericOutputs(
            pixel_values=pixel_values,
            input_ids=prompt_outputs.input_ids,
            attention_mask=prompt_outputs.attention_mask,
            input2_ids=prompt2_outputs.input_ids,
            attention2_mask=prompt2_outputs.attention_mask,
            input3_ids=prompt3_outputs.input_ids,
            attention3_mask=prompt3_outputs.attention_mask,
        )

    def text2image_inputs(
        self,
        prompt: str,
        prompt2: Optional[str] = None,
        prompt3: Optional[str] = None,
        negative_prompt: Optional[str] = "",
        negative_prompt2: Optional[str] = None,
        negative_prompt3: Optional[str] = None,
        max_seq_length: Optional[int] = None,
        max_seq_length2: Optional[int] = None,
    ):
        prompt2 = prompt2 or prompt
        prompt3 = prompt3 or prompt
        negative_prompt2 = negative_prompt2 or negative_prompt
        negative_prompt3 = negative_prompt3 or negative_prompt
        prompt_outputs = self.text_processor1.classification(
            prompt, max_seq_length=max_seq_length
        )
        prompt2_outputs = self.text_processor2.classification(
            prompt2, max_seq_length=max_seq_length
        )
        prompt3_outputs = self.text_processor3.classification(
            prompt3, max_seq_length=max_seq_length2
        )
        negative_prompt_outputs = self.text_processor1.classification(
            negative_prompt, max_seq_length=max_seq_length
        )
        negative_prompt2_outputs = self.text_processor2.classification(
            negative_prompt2, max_seq_length=max_seq_length
        )
        negative_prompt3_outputs = self.text_processor3.classification(
            negative_prompt3, max_seq_length=max_seq_length2
        )
        return GenericOutputs(
            input_ids=prompt_outputs.input_ids,
            attention_mask=prompt_outputs.attention_mask,
            input2_ids=prompt2_outputs.input_ids,
            attention2_mask=prompt2_outputs.attention_mask,
            input3_ids=prompt3_outputs.input_ids,
            attention3_mask=prompt3_outputs.attention_mask,
            negative_input_ids=negative_prompt_outputs.input_ids,
            negative_attention_mask=negative_prompt_outputs.attention_mask,
            negative_input2_ids=negative_prompt2_outputs.input_ids,
            negative_attention2_mask=negative_prompt2_outputs.attention_mask,
            negative_input3_ids=negative_prompt3_outputs.input_ids,
            negative_attention3_mask=negative_prompt3_outputs.attention_mask,
        )

    def image2image_inputs(
        self,
        image: Union[Image.Image, str],
    ):
        if isinstance(image, str):
            image = Image.open(image)
        image = image.convert("RGB")
        if self.image_size is not None:
            image = image.resize(self.image_size)

        pixel_values = self.vae_image_processor.preprocess(image)[0]

        return GenericOutputs(
            pixel_values=pixel_values,
        )

    def controlnet_inputs(
        self,
        image: Union[Image.Image, str],
    ):
        if isinstance(image, str):
            image = Image.open(image)
        image = image.convert("RGB")
        if self.image_size is not None:
            image = image.resize(self.image_size)

        pixel_values = self.vae_condition_image_processor.preprocess(image)[0]
        return GenericOutputs(pixel_values=pixel_values)

    def controlnets_inputs(
        self,
        images: List[Union[Image.Image, str]],
    ):
        pixel_values = []
        for image in images:
            if isinstance(image, str):
                image = Image.open(image)
            image = image.convert("RGB")
            if self.image_size is not None:
                image = image.resize(self.image_size)

            pixel_values.append(self.vae_condition_image_processor.preprocess(image)[0])

        return GenericOutputs(pixel_values=torch.stack(pixel_values, dim=0))

    def adapter_inputs(
        self,
        image: Union[Image.Image, str],
    ):
        if isinstance(image, str):
            image = Image.open(image)
        image = image.convert("RGB")
        if self.image_size is not None:
            image = image.resize(self.image_size)

        pixel_values = self.vae_image_processor.preprocess(image)[0]

        return GenericOutputs(pixel_values=pixel_values)

    def adapters_inputs(
        self,
        images: List[Union[Image.Image, str]],
    ):
        pixel_values = []
        for image in images:
            if isinstance(image, str):
                image = Image.open(image)
            image = image.convert("RGB")
            if self.image_size is not None:
                image = image.resize(self.image_size)

            pixel_values.append(self.vae_image_processor.preprocess(image)[0])

        return GenericOutputs(pixel_values=torch.stack(pixel_values, dim=0))



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import io
import torch
import gc
import gradio as gr
from typing import List, Tuple
from PIL import Image, ImageDraw
from unitorch.cli import CoreConfigureParser, GenericWebUI
from unitorch.cli import register_webui
from unitorch.cli.models.sam import pretrained_sam_infos
from unitorch.cli.pipelines.sam import SamForSegmentationPipeline
from unitorch.cli.webuis import (
    matched_pretrained_names,
    create_element,
    create_accordion,
    create_row,
    create_column,
    create_group,
    create_tab,
    create_tabs,
    create_blocks,
    create_pretrain_layout,
    create_freeu_layout,
)
from unitorch.cli.webuis import SimpleWebUI


@register_webui("core/webui/sam")
class SamWebUI(SimpleWebUI):
    pretrained_names = list(pretrained_sam_infos.keys())
    supported_pretrained_names = matched_pretrained_names(
        pretrained_names,
        "^sam-",
    )

    def __init__(self, config: CoreConfigureParser):
        self._config = config
        self._pipe = None if not hasattr(self, "_pipe") else self._pipe
        self._status = "Stopped" if self._pipe is None else "Running"
        if len(self.supported_pretrained_names) == 0:
            raise ValueError("No supported pretrained models found.")
        self._name = self.supported_pretrained_names[0]

        # create elements
        pretrain_layout_group = create_pretrain_layout(
            self.supported_pretrained_names, self._name
        )
        name, status, start, stop, pretrain_layout = (
            pretrain_layout_group.name,
            pretrain_layout_group.status,
            pretrain_layout_group.start,
            pretrain_layout_group.stop,
            pretrain_layout_group.layout,
        )

        input_image_click = create_element("image", "Input Image")
        mask_threshold_click = create_element(
            "slider", "Mask Threshold", default=0, min_value=-20, max_value=20, step=0.1
        )
        output_image_type_click = create_element(
            "radio", "Output Image Type", default="Mask", values=["Mask", "Object"]
        )
        input_click_reset = create_element("button", "Reset")
        input_click_segment = create_element("button", "Segment")
        input_image_box = create_element("image", "Input Image")
        mask_threshold_box = create_element(
            "slider", "Mask Threshold", default=0, min_value=-20, max_value=20, step=0.1
        )
        output_image_type_box = create_element(
            "radio", "Output Image Type", default="Mask", values=["Mask", "Object"]
        )
        input_box_reset = create_element("button", "Reset")
        input_box_segment = create_element("button", "Segment")
        output_image = create_element("image", "Output Image")

        # create blocks
        click = create_tab(
            create_column(
                input_image_click,
                create_row(mask_threshold_click, output_image_type_click),
                create_row(input_click_reset, input_click_segment),
            ),
            name="Click",
        )
        box = create_tab(
            create_column(
                input_image_box,
                create_row(mask_threshold_box, output_image_type_box),
                create_row(input_box_reset, input_box_segment),
            ),
            name="Box",
        )
        left = create_tabs(click, box)
        right = create_column(output_image)
        iface = create_blocks(pretrain_layout, create_row(left, right))

        # create events
        iface.__enter__()

        start.click(self.start, inputs=[name], outputs=[status])
        stop.click(self.stop, outputs=[status])

        origin_input_image = gr.State(None)
        click_points = gr.State([])
        boxes_points = gr.State([])
        input_image_click.upload(
            lambda image: image.copy() if image is not None else None,
            inputs=[input_image_click],
            outputs=[origin_input_image],
        )
        input_image_click.select(
            self.add_click_points,
            inputs=[origin_input_image, click_points],
            outputs=[input_image_click, click_points],
        )
        input_click_reset.click(
            lambda x: (x, []),
            inputs=[origin_input_image],
            outputs=[input_image_click, click_points],
        )
        input_click_segment.click(
            self.serve_click,
            inputs=[
                origin_input_image,
                click_points,
                mask_threshold_click,
                output_image_type_click,
            ],
            outputs=[output_image],
        )

        input_image_box.upload(
            lambda image: image.copy() if image is not None else None,
            inputs=[input_image_box],
            outputs=[origin_input_image],
        )
        input_image_box.select(
            self.add_click_points,
            [origin_input_image, boxes_points],
            [input_image_box, boxes_points],
        )
        input_box_reset.click(
            lambda x: (x, []),
            inputs=[origin_input_image],
            outputs=[input_image_box, boxes_points],
        )
        input_box_segment.click(
            self.serve_box,
            inputs=[
                origin_input_image,
                boxes_points,
                mask_threshold_box,
                output_image_type_box,
            ],
            outputs=[output_image],
        )

        iface.load(
            fn=lambda: [gr.update(value=self._name), gr.update(value=self._status)],
            outputs=[name, status],
        )

        iface.__exit__()

        super().__init__(config, iname="SAM", iface=iface)

    def start(self, pretrained_name, **kwargs):
        if self._status == "Running":
            self.stop()
        self._name = pretrained_name
        self._pipe = SamForSegmentationPipeline.from_core_configure(
            self._config,
            pretrained_name=pretrained_name,
        )
        self._status = "Running"
        return self._status

    def stop(self, **kwargs):
        self._pipe.to("cpu")
        del self._pipe
        gc.collect()
        torch.cuda.empty_cache()
        self._pipe = None if not hasattr(self, "_pipe") else self._pipe
        self._status = "Stopped" if self._pipe is None else "Running"
        return self._status

    def add_click_points(self, image, click_points, evt: gr.SelectData):
        x, y = evt.index[0], evt.index[1]
        click_points = click_points + [(x, y)]
        new_image = image.copy()
        draw = ImageDraw.Draw(new_image)
        point_color = (255, 0, 0)
        radius = 3
        for point in click_points:
            x, y = point
            draw.ellipse(
                (x - radius, y - radius, x + radius, y + radius), fill=point_color
            )

        return new_image, click_points

    def serve_click(
        self,
        image: Image.Image,
        click_points: List[Tuple[int, int]] = [(0, 0)],
        mask_threshold: float = 0.5,
        output_image_type: str = "Mask",
    ):
        assert self._pipe is not None
        mask = self._pipe(
            image,
            points=click_points,
            mask_threshold=mask_threshold,
        )
        if output_image_type == "Object":
            result = Image.new("RGBA", image.size, (0, 0, 0, 0))
            mask = mask.convert("L").resize(image.size)
            result.paste(image, (0, 0), mask)
        else:
            result = mask
        return result

    def serve_box(
        self,
        image: Image.Image,
        boxes_points: List[Tuple[int, int]] = [(0, 0)],
        mask_threshold: float = 0.5,
        output_image_type: str = "Mask",
    ):
        assert self._pipe is not None
        x1 = min([point[0] for point in boxes_points])
        y1 = min([point[1] for point in boxes_points])
        x2 = max([point[0] for point in boxes_points])
        y2 = max([point[1] for point in boxes_points])
        mask = self._pipe(
            image,
            boxes=[[(x1, y1, x2, y2)]],
            mask_threshold=mask_threshold,
        )
        if output_image_type == "Object":
            result = Image.new("RGBA", image.size, (0, 0, 0, 0))
            mask = mask.convert("L").resize(image.size)
            result.paste(image, (0, 0), mask)
        else:
            result = mask
        return result



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
import torch.nn as nn
from dataclasses import dataclass
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.cli import (
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli import WriterMixin, WriterOutputs
from unitorch.cli.models.modeling_utils import TensorsOutputs, TensorsTargets


@dataclass
class GenerationOutputs(TensorsOutputs, WriterMixin):
    """
    Outputs for generation models.

    Args:
        sequences (torch.Tensor): Generated sequences.
        sequences_scores (Optional[torch.Tensor]): Scores associated with the generated sequences.
            Defaults to an empty tensor.
    """

    sequences: torch.Tensor
    sequences_scores: Optional[torch.Tensor] = torch.empty(0)


@dataclass
class GenerationTargets(TensorsTargets):
    """
    Targets for generation models.

    Args:
        refs (torch.Tensor): Reference sequences.
        masks (Optional[torch.Tensor]): Mask indicating the valid positions in the reference sequences.
            Defaults to an empty tensor.
        sample_weight (Optional[torch.Tensor]): Sample weights for the reference sequences.
            Defaults to an empty tensor.
    """

    refs: torch.Tensor
    masks: Optional[torch.Tensor] = torch.empty(0)
    sample_weight: Optional[torch.Tensor] = torch.empty(0)


class GenerationProcessor:
    def __init__(
        self,
        tokenizer,
    ):
        self.tokenizer = tokenizer

    @classmethod
    @add_default_section_for_init("core/process/generation")
    def from_core_configure(cls, config, **kwargs):
        pass

    @register_process("core/postprocess/generation")
    def _generation(
        self,
        outputs: GenerationOutputs,
    ):
        return WriterOutputs()


def generation_model_decorator(cls):
    class GenerationModel(nn.Module):
        def __init__(self, *args, **kwargs):
            super().__init__()
            if "__generation_model__" in kwargs:
                self.model = kwargs.pop("__generation_model__")
            else:
                self.model = cls(*args, **kwargs)

            __more_attrs__ = [
                "load_state_dict",
                "state_dict",
                "save_checkpoint",
                "from_checkpoint",
                "from_pretrained",
            ]
            for __more_attr__ in __more_attrs__:
                setattr(self, __more_attr__, getattr(self.model, __more_attr__))

            assert hasattr(self.model, "generate")

        def forward(self, *args, **kwargs):
            if self.training:
                return self.model(*args, **kwargs)
            return self.model.generate(*args, **kwargs)

        @classmethod
        def from_core_configure(_cls, cfg, **kwargs):
            model = cls.from_core_configure(cfg, **kwargs)
            return _cls(__generation_model__=model)

    return GenerationModel
